/*-
 * Copyright (c) 2002  Granch Ltd. 	All rigts reserved.
 *
 * All or some portions of this file are derived from material licensed
 * to the Grahcn Ltd. and are reproduced herein with the permission of
 * Granch Ltd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE GRANCH LTD. AND THEIR CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author : Rashid N. Achilov		E-Mail shelton@granch.ru
 *
 *	$Id: hexprint.c,v 0.94 2005/10/04 12:11:54 shelton Exp $
 */
/*-------------------------------------------------------------------------

            Standard functions library

            Print characters line in hex and character forms

            FreeBSD 5.4 version

            Call: void hexprint(char *source, size_t len, char mode);
            Args: source - source string
                  len	 - string length
                  mode   - output mode: SYSLOG - output through syslog() call
                  	                (assumed openlog() was called and LOG_ERR)
                                        PRINTF - output in stdout

            Source string can be finished by \0' or cannot be finished,
            it doesn't matter.

-------------------------------------------------------------------------*/
#include "make.h"

// Main function

void hexprint(char *source, size_t len, char mode)
{
  register int i;
  char *buf,*curptr;

// len * 3 is a hex reperesentation (third byte is a space char), len is a char
// representation, 20 bytes to comments

  buf = (char *) calloc((len * 4) + 20,1);

  sprintf(buf,"hex: ");

// Print in buffer hex form

  for(i = 0,curptr = buf + 4;i <= len;i++,curptr += 3)
   sprintf(curptr,"%.2X ",*(source + i));

// Print in buffer charracter form

  sprintf(curptr,"char: '");

  for(i = 0, curptr += 7;i < len;i++,curptr++)
   sprintf(curptr,"%c",*(source + i));

  sprintf(curptr,"'");

// Output prepared line

  if (mode == SYSLOG)
    syslog(LOG_ERR,"%s",buf);
   else
     puts(buf);

  free(buf);
}
