/*-
 * Copyright (c) 1999
 * The Regents of the University of California, All rights reserved.
 *
 * This code is derived from software contributed to Berkeley by
 * Peter Schweitzer.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

// $Id: itoa.c,v 0.93 2003/08/26 12:56:55 shelton Exp $

#include "make.h"

#define RADIX_MAX 16	/* The radix is usually in the 2 to 16 range */

/*
 * Convert an integer to the corresponding string.
 *
 * Ignores `locale' stuff.  Assumes that the upper and lower case
 * alphabets and digits are each contiguous.
 */

int itoa (int num, char *str, int radix)
{
  register int i, neg = 0;
  register char *p = str;
  register char *q = str;

  if (radix == 0)
    radix = 10;
   else
     if (radix < 2 || radix > RADIX_MAX)
       return (radix);

  if (num == 0)
    {
      *p++ = '0';
      *p = 0;

      return (0);
    }

  if (num < 0)
    {
      neg = 1;
      num = -num;
    }

  while (num > 0)
   {
     i = num % radix;

     if (i > 9)
       i += 7;

     *p++ = '0' + i;
     num /= radix;
   }

  if (neg)
    *p++ = '-';

  *p-- = 0;
  q = str;

  while (p > q)
   {
     i = *q;
     *q++ = *p;
     *p-- = i;
   }

  return (0);
}
