/*-
 * Copyright (c) 2002  Granch Ltd. 	All rigts reserved.
 *
 * All or some portions of this file are derived from material licensed
 * to the Grahcn Ltd. and are reproduced herein with the permission of
 * Granch Ltd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE GRANCH LTD. AND THEIR CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author : Rashid N. Achilov		E-Mail shelton@granch.ru
 *
 *	$Id: libconf.h,v 0.94.2.1 2005/10/06 07:08:22 shelton Exp $
 */
/*--------------------------------------------------------------------------

            Standard functions library

            UNIX FreeBSD 5.4 version

            Common functions and definitions declaration

--------------------------------------------------------------------------*/
#ifndef _LIBCONF_H_
#define _LIBCONF_H_

// Definitions to convert integer to char: conversion to reckon

#define ITOA_HEX	16
#define ITOA_DEC	10
#define ITOA_BIN	 2

// Definitions some characters

#define HZTAB	0x09
#define VTTAB	0x0B
#define SPACE	0x20
#define QUOTA	0x22
#define RETURN	0x0D
#define NEWLINE 0x0A

// Definitions to print character line as hex

#define SYSLOG		1
#define PRINTF		2

// Auto-including library version in generated binary

#define LIBCONF_VERSION_MAJOR	0
#define LIBCONF_VERSION_MINOR  93
#define LIBCONF_VERSION_PATCH   2

#ifdef _LIBCONF_GLOBAL_SET_
unsigned char _libpriv_version_major = LIBCONF_VERSION_MAJOR;
unsigned short int _libpriv_version_minor = LIBCONF_VERSION_MINOR;
unsigned char _libpriv_version_patch = LIBCONF_VERSION_PATCH;
#else
extern unsigned char _libpriv_version_major;
extern unsigned short int _libpriv_version_minor;
extern unsigned char _libpriv_version_patch;
#endif

short int b_atoi(char *in,int border); 		     	 // Conversion to integer
int b_atol(char *in,long int border);	 	     	 // Conversion to long integer
unsigned short b_atoui(char *in,unsigned int border);	 // Conversion to unsigned integer
unsigned int b_atoul(char *in,unsigned long int border); // Conversion to unsigned long integer
int atoh(char *out,char *in,int len);		     	 // Conversion to hex

// Read configuration file
int readconfig(char comment,char *in,char *out,int bsize);

// Parse configuration file
int parseconfig(char *cfgname, struct config_ent *params);

// Copying files: source->source
int fsscopy(char *infile, char *outfile, mode_t mode);

// Copying files: source->source with append to tail
int fssacopy(char *infile, char *outfile, mode_t mode);

// Copying files: file descriptor->source
int fbscopy(FILE *infile, char *outfile, mode_t mode);

// Copying files: file descriptor->source with append to tail
int fbsacopy(FILE *infile, char *outfile, mode_t mode);

// Copying files: source->file descriptor
int fsbcopy(char *infile, FILE *outfile, mode_t mode);

// Copying files: source->file descriptor with append to tail
int fsbacopy(char *infile, FILE *outfile, mode_t mode);

// Copying files: file descriptor->file descriptor
int fbbcopy(FILE *infile, FILE *outfile, mode_t mode);

// Copying files: file descriptor->file descriptor with append to tail
int fbbacopy(FILE *infile, FILE *outfile, mode_t mode);

int encode_base64(char *src, char *dest, int szdest,int srclen);
int decode_base64(char *src, char *dest, int szdest);
void replacestring(char **src,char *part1,char *part2,int *ssize);
                                                 // Replace string part by another one
int _parsedate(struct tm *ptm,char *textdata);	 // Date parsing
int _parsetime(struct tm *ptm,char *textdata);	 // Time parsing
int itoa(int num, char *str, int radix);	 // Conversion integer to string
void strunescape(char *source, char symbol);	 // Unescape line againts char
void strnunescape(char *source, char symbol,size_t len); // Unescape line againts char

// Convert strings from UPPER to lower case and from lower to UPPER case
char *strtoupper(char *to, const char *from);	// Convert string to UPPER case
char *strtolower(char *to, const char *from);	// Convert string to lower case

// Convert strings from UPPER to lower case and from lower to UPPER case
// Convert not more than n bytes
char *strntolower(char *dst, const char *src, size_t n); // Convert n bytes from str
char *strntoupper(char *dst, const char *src, size_t n); // Convert n bytes from str

// Compare strings not more than len bytes, ignore case
char *strncasestr(register const char *s, register const char *find, size_t len);
void hexprint(char *source, size_t len, char mode); // Print character line as hex

// A safe compare not more than len bytes
char *__strnstr(const char *s, const char *find, size_t slen);

// Checking a presence option in options list
int _checklist(char *option, char *binopt, char **list);

#endif
