/*-
 * Copyright (c) 2002  Granch Ltd. 	All rights reserved.
 *
 * All or some portions of this file are derived from material licensed
 * to the Grahcn Ltd. and are reproduced herein with the permission of
 * Granch Ltd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE GRANCH LTD. AND THEIR CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author : Rashid N. Achilov		E-Mail shelton@granch.ru
 *
 *	$Id: parseconfig.c,v 1.1.2.1 2005/10/06 11:54:41 shelton Exp $
 */
/*-------------------------------------------------------------------------

            Standard functions library

            Parsing configuration file.

	    This is mostly difficult for understood function. It was in
            closed-state since 1997, but now I have enough expirience to
            make in independent and include it in library. It parses
            configuration files with typical "variable=value" lines, where
            "variable" can be string, decimal, boolean and list.
            See man parseconfig to more info.

 	    UNIX FreeBSD 5.4 version


	    Call: int parseconfig(char *config, config_ent[] *params);

	    Call with: config - pointer to configuration file name
            	       params - array of config_ent structures, describing
                       		each configuration file element. See
                                parseconfig.h to description

	    Return with:  NO - parsing error, message arrived
            		  OK - parsing OK

	    Maximun length of string or list - 255 chars
	    Maximum of digits in non-char variables - 9

	    First TWO parameters in parameters list MUST be string!!

            In description scructure all string (STDCHAR, STDLIST, STDARRAY)
            parameters MUST be grouped _BEFORE_ any int or bool!

-------------------------------------------------------------------------*/
#include "make.h"
#include "parseconfig.h"

int parseconfig(char *cfgname, struct config_ent *params)
{
  short int i, chparm, nochparm;
  char *ids, *pos, *line, *buf, *bufx, *posix, *nafig, sig = 0;
  char *prevrear;
  char **pts;
  int work, retcode = 0, bla;
  struct stat sb;
  struct config_ent *tmpent3 = params, *tmpent2;

// Detect quantity of parameters, quantity of string and non-string
// parameters to memory allocation

  for(chparm = nochparm = 0, tmpent2 = params; tmpent2->exit != NULL;tmpent2++)
   if ((tmpent2->exit == STDCHAR) ||
   		(tmpent2->exit == STDLIST) || (tmpent2->exit == STDARRAY))
     chparm++;
    else
      nochparm++;

// Allocate memory to config file string variables and local area

  if ((*(char **) tmpent3->which =
//    if ((pts = (char **) calloc((MAXLEN * chparm), 1)) == NULL)
   		 	(char *) calloc((MAXLEN * chparm), 1)) == NULL)
    {
      fprintf(stderr,"malloc error, out of memory : %s\n",strerror(errno));
      return NO;
    }
//    else
//      *((char **) tmpent3->which) = pts;

// Allocation memory to all local and string-variables-config-file areas
// Allocation done from 'ids' through 'nafig' and than all string-variables
// (if any).

  for(pts = (char **) (tmpent3 + 1)->which, i = 0,
  		bufx =  *((char **) tmpent3->which) + MAXLEN;
  			   	i < (chparm - 1);i++, pts++, bufx += MAXLEN)
   *pts = bufx;

// Initialize all non-char parameters

  for(tmpent2 = params, i = 0;i < (chparm + nochparm);i++, tmpent2++)
   {
     if ((tmpent2->exit != STDCHAR) &&
   		(tmpent2->exit != STDLIST) && (tmpent2->exit != STDARRAY))
       switch(tmpent2->exit)
        {
          case STDBOOL:
          case STDHEX1: *((unsigned char *) tmpent2->which) = 0x0;
          		 break;

          case STDSHORT:
          case STDUSHORT:
          case STDHEX2: *((unsigned short int *) tmpent2->which) = 0x0;
          		 break;

          case STDINT:
          case STDUINT:
          case STDUID:
          case STDGID:
          case STDHEX4: *((unsigned int *) tmpent2->which) = 0x0;
          		 break;

	  case STDLONG:
          case STDULONG:
          case STDHEX8: *((unsigned long int *) tmpent2->which) = 0x0;
          		 break;
        }
   }

// Allocating workspace

  ids = (char *) calloc(MAXLEN + 3, 1);
  nafig = ids + MAXLEN;
  prevrear = nafig + MAXLEN;

// Open config file

  if ((work = open(cfgname,O_RDONLY)) > 0)
    {
      stat(cfgname, &sb);
      bufx = (char *) malloc(sb.st_size * 2);
      buf = bufx + sb.st_size;

      read(work, bufx, sb.st_size);	// Fucked Feof()!
      close(work);

// Now we drop lines with comment char at first position and strip spaces.
// After this procedure in output buffer lies only name=value pairs, separa-
// ted by '\n' char

      if ((bla = readconfig('#',bufx,buf,sb.st_size)) == ERR)
	{
	  fprintf(stderr,"cannot read configuration file %s\n",cfgname);
          free(ids);
          return NO;
	}

// Reading filtered buffer line-by-line. Now we search '\n' char and at each
// loop round shift position on 1 greater than searched position. Loop stops
// when length filtered buffer will be exhausted.

      for(line = buf;(line - buf) < bla;line = posix + 1,sig = NULL)
       {
	 posix = strchr(line,'\n');

// Search '=' char and taking string BEFORE it as parameter name

	 pos = strchr(line,'=');

	 bzero(ids,32);
	 strncpy(ids, line, pos - line);

// Search in config file parameters array this parameter. Search stops when
// parameter will be found or parameters quantity will expired. When
// search flag is on, search terminate immediately.

         for(tmpent2 = params, i = 0;(i < (chparm + nochparm)) && (!sig);i++,tmpent2++)
          if (!(strcmp(ids,tmpent2->name)))
            {
              switch(tmpent2->exit)
               {

// STDINT and similar processors take a value from string target and convert it to
// desired integer (signed or unsigned) value. Cehck on default value and integrate
// when exist

                 case STDSHORT: if ((*(short int *) tmpent2->which =
                 		   	    b_atoi(pos + 1, SSMAX)) >= NO)
	       		          sig = tmpent2->valid = OK;
                                 else
                                   if (tmpent2->defvalue)
                 		     if ((*(short int *) tmpent2->which =
                 		       	   b_atoi(tmpent2->defvalue, SSMAX)) >= NO)
	       		             sig = tmpent2->valid = OK;
                                      else
                                        tmpent2->valid = ERR;
                                    else
                                      tmpent2->valid = ERR;

	     		        continue;

                 case STDUSHORT: if ((*(unsigned short int *) tmpent2->which =
                 		   	              b_atoui(pos + 1, USMAX)) >= NO)
	       		           sig = tmpent2->valid = OK;
                                  else
                                    if (tmpent2->defvalue)
                 		      if ((*(short int *) tmpent2->which =
                 		   	    b_atoui(tmpent2->defvalue, USMAX)) >= NO)
	       		                sig = tmpent2->valid = OK;
                                       else
                                         tmpent2->valid = ERR;
                                     else
                                       tmpent2->valid = ERR;

	     		         continue;

                 case STDINT: if ((*(int *) tmpent2->which =
                 		   	    b_atol(pos + 1, SLMAX)) >= NO)
	       		        sig = tmpent2->valid = OK;
                               else
                                 if (tmpent2->defvalue)
                 		   if ((*(int *) tmpent2->which =
                 		         b_atol(tmpent2->defvalue, SLMAX)) >= NO)
	       		             sig = tmpent2->valid = OK;
                                    else
                                      tmpent2->valid = ERR;
                                  else
                                    tmpent2->valid = ERR;

	     		      continue;

                 case STDUINT: if ((*(unsigned int *) tmpent2->which =
                 		   	    b_atoul(pos + 1, ULMAX)) >= NO)
	       		         sig = tmpent2->valid = OK;
                                else
                                  if (tmpent2->defvalue)
                 		    if ((*(int *) tmpent2->which =
                 		         b_atoul(tmpent2->defvalue, ULMAX)) >= NO)
	       		              sig = tmpent2->valid = OK;
                                     else
                                       tmpent2->valid = ERR;
                                   else
                                     tmpent2->valid = ERR;

	     		       continue;

// Processor GID parameter takes GID and stores gid_t type

		 case STDGID: if ((*(gid_t *) tmpent2->which =
                 		   	    b_atoui(pos + 1, NOGROUP)) >= NO)
	       		        sig = tmpent2->valid = OK;
                               else
                                 if (tmpent2->defvalue)
                 		   if ((*(int *) tmpent2->which =
                 		         b_atoui(tmpent2->defvalue, USMAX)) >= NO)
	       		             sig = tmpent2->valid = OK;
                                    else
                                      tmpent2->valid = ERR;
                                  else
                                    tmpent2->valid = ERR;

	     		       continue;

// Processor UID parameter takes UID and stores uid_t type

		 case STDUID: if ((*(uid_t *) tmpent2->which =
                 		   	    b_atoui(pos + 1, NOGROUP)) >= NO)
	       		        sig = tmpent2->valid = OK;
                               else
                                 if (tmpent2->defvalue)
                 		   if ((*(int *) tmpent2->which =
                 		         b_atoui(tmpent2->defvalue, USMAX)) >= NO)
	       		             sig = tmpent2->valid = OK;
                                    else
                                      tmpent2->valid = ERR;
                                  else
                                    tmpent2->valid = ERR;

	     		       continue;

// Processors STDHEXN translated textual representation of hexadecimal values in
// their real values, depend on length - 1, 2, 4 or 8 bytes. Here exit code
// using as length!

                 case STDHEX1:
		 case STDHEX2:
                 case STDHEX4:
		 case STDHEX8: if (!atoh((unsigned char *) tmpent2->which,
                 					pos + 1, tmpent2->exit))
                                 sig = tmpent2->valid = ERR;
                                else
                                  tmpent2->valid = OK;

                               continue;

// Processor type STDCHAR processed
// string value through copying it to address from configParam[i].which
// pointer. Length of copying string calculated as difference between
// '\n' char and '=' char (and less to 1). Processor type STDINT processed

                 case STDCHAR: strncpy(*((char **) tmpent2->which),
                 				  pos + 1,posix - pos - 1);

	       		       sig = tmpent2->valid = OK;
		 	       continue;

// STDBOOL processor intended to create unary bit flags, based on 'yes/no' config
// parameter. In paramConfig[i] hexmask field in inportant for this processor!
// Accorded by them, flags byte will changed to set/unset flag, accorded by parameter
// value

   	         case STDBOOL: bzero(nafig,4);
   	         	       strncpy(nafig, pos + 1, posix - pos - 1);

                 	       if (!strcasecmp(nafig,"YES"))
                                 *((char *) tmpent2->which) ^=
                                 		  tmpent2->hexmask;

                               sig = tmpent2->valid = OK;
                 	       continue;

// STDLIST parameter: check op presence option, specified in config file in
// options list with parameter 'list'. When option was not found or error - error
// state

   	         case STDLIST: bzero(nafig,8);
   	         	       strncpy(nafig, pos + 1, posix - pos - 1);

                               if (_checklist(nafig,
            			    (char *) tmpent2->which, tmpent2->list))
	       		         sig = tmpent2->valid = OK;
                                else
                                  tmpent2->valid = ERR;

                 	       continue;

// STDLONG and STDULONG processors didn't implemented yet

                 case STDLONG:
                 case STDULONG: fputs("Sorry, not implemented yet",stderr);
                 		free(ids);
                 		return NO;

                 default:      fprintf(stderr,"unknown processor exit code %d\n",
                 				       		      tmpent2->exit);
                               free(ids);
                               return NO;
               }
            }
       }

      free(bufx);

// Check on presence default values and integrate, when exist and accessed
      for(tmpent2 = params, i = 0;i < (chparm + nochparm);i++,tmpent2++)
       {

// Checkout all parsed fields
         if (tmpent2->valid == NO)

// Integrate default value
           if (tmpent2->defvalue != NULL)
             {
               if (tmpent2->exit == STDCHAR)
                 {
                   strncpy(*((char **) tmpent2->which),
                            tmpent2->defvalue, strlen(tmpent2->defvalue));
                   tmpent2->valid = OK;
                 }
                else
                  fprintf(stderr,"Default value for parameter %s is missed\n",
                     						   tmpent2->name);
             }
       }			// for

// Check on presence all parameters
      for(tmpent2 = params, i = 0;i < (chparm + nochparm);i++, tmpent2++)
       {
         if (tmpent2->valid == NO)
           {
             fprintf(stderr,
           	"parameter %s invalid: %s\n",
           		tmpent2->name, (char *) tmpent2->which);
             retcode = ERR;
           }
       }

// When errors in split config file were detected, stop processing
      if (retcode == ERR)
	{
	  fputs("Configuration file parsing error detected",stderr);
          free(ids);
          return NO;
	}
    }

// Specified configuration file didn't found
     else
       {
	 fprintf(stderr,"opening file %s error: %s\n",cfgname,strerror(errno));
         return NO;
       }

// Free allocated workspace
  free(ids);
  return OK;
}
