/*-
 * Copyright (c) 2002  Granch Ltd. 	All rigts reserved.
 *
 * All or some portions of this file are derived from material licensed
 * to the Grahcn Ltd. and are reproduced herein with the permission of
 * Granch Ltd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE GRANCH LTD. AND THEIR CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author : Rashid N. Achilov		E-Mail shelton@granch.ru
 *
 *	$Id: parseconfig.h,v 0.94.2.1 2005/10/06 07:08:22 shelton Exp $
 */
/*-------------------------------------------------------------------------

            Standard functions library

 	    UNIX FreeBSD 5.4 version

	    Descriptions to parsing configuration file

-------------------------------------------------------------------------*/
#ifndef _PARSE_CONFIG_
#define _PARSE_CONFIG_

// Very common definitions

#define OK                1               // Good result
#define NO                0               // Bad result
#define ERR              -1               // Error indicated
#define NUL		  0		  // Zero

// Common parameters to process config files

#define MAXLEN		256		  // Maximum identifier length (+1 ZSB)
					  // in config file
#define MAXBLOCK       1024		  // Maximum work area size (+1 ZSB)
#define MAXDIGIT	  9		  // aximum digits in one value

#define USMAX	      65534		  // Maximum unsigned short value
#define SSMAX	      32766		  // Maximum signed short value
#define ULMAX         4294967294	  // Maximim unsigned int value
#define SLMAX         2147483646          // Maximum signed int value

// This is a configuration file parameters structure description. It has
// next fields:
// parameter name (up to 32 chars, use for search parameter)
// parameter processor code (one char, use for search valid parameter
// 			     processor and processing parameter value)
// parameter validity sign (one char, use for check parameter validity,
// 			    set by processor, check after processing all file)
// default value (in STDLIST - pointer to pointer to list elements)
// destination address (see below)
// For character lines
// as destination address set a pointer to pointer to allocated area, for
// other types pointer to corresponding object. For bool parameter destina-
// tion address will be pointed to a whole bit set.

struct config_ent
 {
   char name[32]; 		// Parameter name
   unsigned char exit;		// Parameter processor code
   unsigned char valid;		// Sign of validity
   unsigned char hexmask;	// When processor is STDBOOL this is a mask
   				// to set right bit in bit set, else ignored
   char **list;			// Pointer to pointer to options list, used in STDLIST
   char *defvalue;		// Default value for this parameter (if any)
   void *which; 		// Destination address
 };

// Parameters processors

#define STDHEX1		  1		  // hexadecimal 1 byte
#define STDHEX2		  2		  // hexadecimal 2 bytes
#define STDSHORT	  3		  // short (2 bytes) integer
#define STDHEX4		  4		  // hexadecimal 4 bytes
#define STDLONG		  5		  // long integer
#define STDUINT		  6		  // unsigned integer
#define STDULONG 	  7		  // unsigned long integer
#define STDHEX8		  8		  // hexadecimal 8 bytes
#define STDGID		  9		  // GID
#define STDARRAY 	 10		  // lines array
#define STDLIST		 11		  // options list
#define STDCHAR		 12		  // character line
#define STDINT		 14		  // integer
#define STDBOOL		 15		  // bool (yes/no)
#define STDUID		 16		  // UID
#define STDUSHORT	 17		  // unsigned short int (2 bytes)

#endif
