/*-
 * Copyright (c) 2002  Granch Ltd. 	All rigts reserved.
 *
 * All or some portions of this file are derived from material licensed
 * to the Grahcn Ltd. and are reproduced herein with the permission of
 * Granch Ltd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE GRANCH LTD. AND THEIR CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author : Rashid N. Achilov		E-Mail shelton@granch.ru
 *
 *	$Id: parsedate.c,v 0.94 2005/10/04 12:11:54 shelton Exp $
 */
/*-------------------------------------------------------------------------

            Standard functions library

            Parsing of text data field into struct tm fields

            FreeBSD 5.4 version

            Call: int _parsedate(struct tm *ptm, char *textdata);
            Source string must be in format DD/MM/YYYY, where as '/'
            can be any separator char

            Return:  1 - data parsed succesfully
                     0 - source data string invalid

-------------------------------------------------------------------------*/
#include "make.h"

// Local data structure description

struct indate    				// Source string
 {
   char inday[2];				// Day
   char sep1;
   char inmonth[2];				// Month
   char sep2;
   char inyear[4];				// Year
 };

// Main function

int _parsedate(struct tm *ptm,char *textdata)
{
  struct indate *indptr = (struct indate *) textdata;

  ptm->tm_mday = atoi(indptr->inday);            // Convert source day

  if ((ptm->tm_mday < 1) || (ptm->tm_mday > 31)) // Check on validity
    return NO;

  ptm->tm_mon = atoi(indptr->inmonth) - 1;       // Convert source month

  if ((ptm->tm_mon < 0) || (ptm->tm_mon > 11))   // Check on validity
    return NO;

  ptm->tm_year = atoi(indptr->inyear) - 1900;    // Convert source year

  return OK;
}
