/*-
 * Copyright (c) 2002  Granch Ltd. 	All rigts reserved.
 *
 * All or some portions of this file are derived from material licensed
 * to the Grahcn Ltd. and are reproduced herein with the permission of
 * Granch Ltd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE GRANCH LTD. AND THEIR CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author : Rashid N. Achilov		E-Mail shelton@granch.ru
 *
 *	$Id: parsetime.c,v 0.94 2005/10/04 12:11:54 shelton Exp $
 */
/*-------------------------------------------------------------------------

            Standard functions library

            Parsing of text time field into struct tm fields

            FreeBSD 5.4 version

            Call: int _parsetime(struct tm *ptm, char *textdata);
            Source string must be in format HH/MM/SS, where as '/'
            can be any separator char

            Return:  1 - time parsed succesfully
                     0 - source data string invalid

-------------------------------------------------------------------------*/
#include "make.h"

// Local data types description

struct intime    				// Source string structure
 {
   char inhour[2];				// Hour
   char sep1;
   char inmin[2];				// Minute
   char sep2;
   char insec[4];				// Sec
 };

// Main function


int _parsetime(struct tm *ptm,char *textdata)
{
  struct intime *intptr = (struct intime *) textdata;

  ptm->tm_hour = atoi(intptr->inhour);           // Convert hour

  if ((ptm->tm_hour < 0) || (ptm->tm_hour > 23)) // Check on validity
    return NO;

  ptm->tm_min = atoi(intptr->inmin);             // Convert minute

  if ((ptm->tm_min < 0) || (ptm->tm_min > 59))   // Check on validity
    return NO;

  ptm->tm_sec = atoi(intptr->insec);             // Convert sec

  if ((ptm->tm_sec < 0) || (ptm->tm_sec > 59))   // Check on validity
    return NO;

  return OK;
}
