/*-
 * Copyright (c) 2002    Granch Ltd. 	All rights reserved.
 *
 * All or some portions of this file are derived from material licensed
 * to the Grahcn Ltd. and are reproduced herein with the permission of
 * Granch Ltd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE GRANCH LTD. AND THEIR CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author : Rashid N. Achilov 		E-Mail shelton@sentry.granch.ru
 *
 *	$Id: localtypes.h,v 0.95 2005/10/04 12:11:54 shelton Exp $
 */
/*-------------------------------------------------------------------------

            Spam Control and Canceling mail filter, based on libmilter API

	    UNIX FreeBSD 5.4 version

            Global types definition.

-------------------------------------------------------------------------*/

// Mode byte for processing modes as bit field structure
struct _mode
 {
   unsigned 	 _discard		:1;	// Discard message
   unsigned	 _reject		:1;	// Reject message
   unsigned 	 _isolate		:1;	// Isolate mode
   unsigned	 _res			:5;	// Reserved bits
 };

// Isolation mode byte as bit field structure
struct _hmode
 {
   unsigned 	 _incoming		:1;	// Incoming messages
   unsigned	 _outgoing		:1;	// Outgoing messages
   unsigned	 _res			:6;	// Reserved bits
 };

// Local flags structure (one global data)
struct local_flags
 {
   unsigned	 _openlog		:1;	// System log is opened
   unsigned	 _pidfile		:1;	// PID file is presented
   unsigned	 _alloc_config  	:1;	// Memory to config file allocated
   unsigned	 _imitate		:1;	// Imitate only

   unsigned	 _nodelete		:1;	// No delete any mail anyway
   unsigned	 _spam_nodelete		:1;	// No delete spam mail anyway
   unsigned	 _res1			:1;	// Reserved bit
   unsigned	 _res2			:1;	// Reserved bit
 };

// Connection flag structure (one per connect)
struct conn_flags
 {
   unsigned	_isspam		:1;	// This mail is spam-suspected
   unsigned 	_firstheader	:1;	// First header line already was
   unsigned	_tempsave	:1;	// Open temp/save header file
   unsigned	_firstrcpt	:1;	// First recipient already was given
//
   unsigned	_isenterviol	:1;	// This mail violated enterprise rules
   unsigned	_htmlfrom	:1;	// Allow to receive HTML messages
   unsigned	_localout	:1;	// Local user mail out
   unsigned	_multipart	:1;	// This is a multipart message
//
   unsigned	_charset	:1;	// charset= was found in Content-Type
   unsigned	_boundary	:1;	// boundary= was found in Content-Type
   unsigned	_bstop		:1;	// boundary stopper was found
   unsigned	_noctparam	:1;	// No parameters followed after media type
//
   unsigned	_tempbody	:1;	// Open temp/save body file
   unsigned	_dbgsave	:1;	// Saved for future investigate
   unsigned	_attachfrom	:1;	// Allow to receive attachments
   unsigned	_res4		:1;	// Reserved bit
 };

// Boundary flag structure (one per boundary)
struct bnd_flags
 {
   unsigned	_noctparam	:1;	// Content-Type has only type/subtype
   unsigned	_image		:1;	// Media type is image
   unsigned	_audio		:1;	// Media type is audio
   unsigned	_video		:1;	// Media type is video
//
   unsigned	_application	:1;	// Media type is application
   unsigned	_multipart	:1;	// This boundary contains multipart message
   unsigned	_boundary	:1;	// boundary= was found
   unsigned	_text		:1;	// Media type is text
//
   unsigned	_incomplete	:1;	// MIME header of boundary was not finished
   unsigned	_iscontype	:1;	// Content-Type was not finished yet
   unsigned	_media		:1;	// Media type was parsed
   unsigned	_res12		:1;	// Reserved bit
//
   unsigned	_res13		:1;	// Reserved bit
   unsigned	_res14		:1;	// Reserved bit
   unsigned	_res15		:1;	// Reserved bit
   unsigned	_res16		:1;	// Reserved bit
 };

// Sender parameters structure (one per sender)
struct send_flags
 {
   unsigned	_htmlin		:1;	// Allow to receive HTML messages
   unsigned	_htmlout	:1;	// Allow to send HTML messages
   unsigned	_pgp		:1;	// Allow to send/receive PGP encrypted mail
   unsigned     _exattach	:1;	// Allow to receive executable attachments

   unsigned	_attach		:1;	// Allow to receive any other attachments
   unsigned	_ischarset	:1;	// This sender has defined charset table
   unsigned			:2;	// Reserved bits
 };

// Receiver parameters structure (one per receiver)
struct recv_flags
 {
   unsigned	_htmlin		:1;	// Allow to receive HTML messages
   unsigned	_htmlout	:1;	// Allow to sendL messages
   unsigned	_pgp		:1;	// Allow to send/receive PGP encrypted mail
   unsigned     _exattach	:1;	// Allow to receive executable attachments

   unsigned	_attach		:1;	// Allow to receive any other attachments
   unsigned	_ischarset	:1;	// This recipient has defined charset table
   unsigned			:2;	// Reserved bits
 };

// Hosts parameters structure (one per host from scchosts)
struct host_flags
 {
   unsigned	_html		:1;	// Allow to receive HTML messages
   unsigned	_attach		:1;	// Allow to receive attachments
   unsigned			:6;	// reserved bits
 };

// User parameters structure (one per user from sccusers)
struct user_flags
 {
   unsigned	_htmlin		:1;	// Allow to receive HTML messages
   unsigned	_htmlout	:1;	// Allow to send HTML messages
   unsigned	_pgp		:1;	// Allow to send/receive PGP encrypted mail
   unsigned     _exattach	:1;	// Allow to receive executable attachments
//
   unsigned	_attach		:1;	// Allow to receive any other attachments
   unsigned	_ischarset	:1;	// This recipient has defined charset table
   unsigned	_shadow		:1;	// Mail from this user will shadowed
   unsigned	_res2		:1;	// Reserved bits
 };

struct mlfiPriv				// Mail filter private data storage
 {
   char    	     *mlfi_fname;	// Temp/save header filename
   char 	     *mlfi_bname;	// Temp/save body filename
   char    	     *mlfi_connectfrom; // Connect from: container
   struct sccuser    *mlfi_mailfrom;	// MAIL FROM: container
   char    	     *mlfi_helofrom;	// HELO from: container
   struct sccuser    *mlfi_rcptto;	// RCPT TO:'s container
   FILE    	     *mlfi_fp;	    	// File handle to keep header
   FILE		     *mlfi_fpbody;	// File handle to keep body
   struct in_addr    inaddr;		// Host IP in hex form (4 bytes)
   struct tm 	     *ptm;		// Time containver
   struct conn_flags _cflags;		// Connection flags
   struct bnd_flags  _bflags;		// Boundary flags
   short int         recipients;	// Number of recipients in mail
   short int	     headerlines;	// Header lines counter
   short int	     body_parts;	// Mail body can be split in a chunks
   int		     body_length;	// Message body (accomplished) length
   short int	     charsetlength;	// Length of value in charset= parameter
   short int	     boundarylength;	// Length of value in boundary= parameter
   char		     *work;		// Workspace to codepages, etc.
   char		     *charset;		// Pointer to charset storage area
   char		     *boundary;		// Pointer to boundary copy
 };

// Configuration file flags structure (one global)
struct config_flags
 {
   unsigned _isdaemon		:1;  	// Go to daemon mode
   unsigned _res1		:1;  	// Reserved
   unsigned _res2		:1;  	// Reserved
   unsigned _res3	 	:1;	// Reserved
//
   unsigned _res4	  	:1;	// Reserved
   unsigned _res5		:1;	// Reserved
   unsigned _enterprise		:1;	// Enterprise violations control
   unsigned _nocharset		:1;	// No charset issued spam action
 };

// Configuration file itself structure
struct local_config
 {
   char *progname;			// Mandatory field - program name
   char *progversion;			// Mandatory field - program version
   char *sendmail_pipe;			// Pipe to use with sendmail
   char *scchosts;			// SCC Hosts file
   char *sccusers;			// SCC users file
   char *pidfile;			// PID number keeper
   char *isolation;			// Directory to keep isolated spam mail
   char *extrasave;			// Directory to keep extra saved mail
   char *codepages;			// Allowed codepages
   char *domain;			// Local domain parameter
   short int sendmail_timeout;		// Timeout to connect to sendmail
   short int debug_level;		// Level of debugging messages
   struct config_flags _idflags;	// Config file flags
   struct _mode mode;			// Mode for spam mail
   struct _mode emode;	  		// Mode for enterprise violated mail
   struct _hmode hmode; 		// HTML isolation mode
 };

// Record from scchosts file structure (one per host)
struct scchost
 {
   struct in_addr scc_addr;		// Host IP in hex form (4 bytes)
   struct host_flags _hflags;		// Host flags
 };

// Record from sccusers file structure (one per user)
struct sccuser
 {
   char sccuser[81];		  	// Username (max 80 chars + zero pad)
   struct user_flags _uflags;		// User flags
   char sccpages[61];			// This user's allowed codepages
 };
