/*-
 * Copyright (c) 2002  Granch Ltd. 	All rights reserved.
 *
 * All or some portions of this file are derived from material licensed
 * to the Grahcn Ltd. and are reproduced herein with the permission of
 * Granch Ltd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE GRANCH LTD. AND THEIR CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author : Rashid N. Achilov		E-Mail shelton@granch.ru
 *
 *	$Id: mlficatch.c,v 0.96.2.2 2005/10/06 10:24:22 shelton Exp $
 */
/*------------------------------------------------------------------------

        Spam Control and Canceling mail filter, based on libmilter API

 	UNIX FreeBSD 5.4 version

        Mail Filter Catch-up callbacks

--------------------------------------------------------------------------*/
#include "sccmilter.h"
#include "externals.h"
#include "functions.h"

/* ---------------------------------------------------------------------------------
 *
 *	connection catch-up (connection to sendmail daemon was detected)
 *
 *	Call with:	ctx  	 - communications area for sendmail
 *			hostname - pointer to connected hostname
 *				   (resolved by sendmail)
 *			hostaddr - _SOCK_ADDR pointer, keeping IP address of
 *				   connected host, so we do not need backresolv
 *
 *	Return code:	SMFIS_CONTINUE	- continue message processing
 *			SMFIS_TEMPFAIL	- fatal error was detected, abort message
 *
 * --------------------------------------------------------------------------------- */

sfsistat mlfi_connect(SMFICTX *ctx, char *hostname, _SOCK_ADDR *hostaddr)
{
  struct mlfiPriv *priv;
  char *localid = "(connect)";
  struct scchost *scc1;
  struct sockaddr_in *lp;
  short int i;

// Allocate memory to mlfiPriv data structure

  if ((priv = calloc(sizeof (struct mlfiPriv),1)) == NULL)
    {
      syslog(LOG_ERR,"%s mlfiPriv %s, failed",cannot,localid);
      return SMFIS_TEMPFAIL;
    }

// Set private data area address

  smfi_setpriv(ctx, (void *) priv);

// Save CONNECT From: in container

  if ((priv->mlfi_connectfrom = strdup(hostname)) == NULL)
    {
      syslog(LOG_ERR,"%s hostname %s, failed",cannot,localid);
      return SMFIS_TEMPFAIL;
    }

// Store an address in structure for future use

  lp = (struct sockaddr_in *) hostaddr;
  priv->inaddr.s_addr = lp->sin_addr.s_addr;

#ifdef DBPRINT
// Here we dump SCC hosts table

  if (_SCC_milter_config.debug_level > 50)
    {
      syslog(LOG_ERR,"%s %s SCC hosts table (all is %d hosts)",
      						localid,dblev[1],qscchosts);

      for(i = 1, scc1 = scchosts;i <= qscchosts;i++,scc1++)
       syslog(LOG_ERR,"%s %s scc->scc_addr=0x%X (%s),scc->_hflags=0x%X",
              		localid,dblev[1],*((u_int *) &scc1->scc_addr.s_addr),
              		inet_ntoa(scc1->scc_addr),*((u_char *) &scc1->_hflags));
    }
#endif

// Walk through SCC hosts file and check, if connected host allowed to send HTML
// messages

  for(i = 1, scc1 = scchosts;i <= qscchosts;i++,scc1++)
   if ((bcmp(&scc1->scc_addr.s_addr,&priv->inaddr.s_addr,4)) == 0)
     {

#ifdef DBPRINT
       if (_SCC_milter_config.debug_level > 20)
         syslog(LOG_ERR,"%s %s We found host %s in SCC hosts table",
       					localid,dblev[0],inet_ntoa(priv->inaddr));
#endif
// Here we on correspond flags in _cflags, when there is on in _hflags
// HTML allowed

       if (scc1->_hflags._html)
         {
           priv->_cflags._htmlfrom = OK;
#ifdef DBPRINT
           if (_SCC_milter_config.debug_level > 50)
             syslog(LOG_ERR,
             	"%s %s HTML allowed bit was set, priv->_cflags=0x%X",
             			localid,dblev[1],*((u_short *) &priv->_cflags));
#endif
         }

// Attaches allowed

       if (scc1->_hflags._attach)
         {
           priv->_cflags._attachfrom = OK;
#ifdef DBPRINT
           if (_SCC_milter_config.debug_level > 50)
             syslog(LOG_ERR,
             	"%s %s Attaches allowd bit was set, priv->_cflags=0x%X",
             			localid,dblev[1],*((u_short *) &priv->_cflags));
#endif
         }

       break;
     }

#ifdef DBPRINT
  if (i > qscchosts)
    if (_SCC_milter_config.debug_level > 20)
      syslog(LOG_ERR,"%s %s Host %s does not found in SCC hosts table",
    					localid,dblev[0],inet_ntoa(priv->inaddr));

  if (_SCC_milter_config.debug_level > 50)
    syslog(LOG_ERR,"%s %s startup idflags=0x%X",localid,dblev[1],
   				*((u_char *) &_SCC_milter_config._idflags));
#endif

// Walk through...

  return SMFIS_CONTINUE;
}

/* ---------------------------------------------------------------------------------
 *
 *	HELO catch-up (HELO command was arrived)
 *
 *	Call with:	ctx  	 - communications area for sendmail
 *			helohost - pointer to specified HELO host name
 *
 *	Return code:	SMFIS_CONTINUE	- continue message processing
 *			SMFIS_TEMPFAIL	- fatal error was detected, abort message
 *
 * --------------------------------------------------------------------------------- */

sfsistat mlfi_helo( SMFICTX *ctx, char *helohost)
{
  char *tls;
  char *buf;
  char *localid = "(helo)";
  struct mlfiPriv *priv = MLFIPRIV;

// Try to take TLS version

  tls = smfi_getsymval(ctx, "{tls_version}");

  if (!tls)
    tls = "No TLS";

// Try to take HELO (if any)

  if (!helohost)
    helohost = "(no HELO issued)";

// Allocate container to HELO

  if (!(buf = (char *) malloc (strlen(tls) + strlen (helohost) + 3)))
    {
      syslog(LOG_ERR,"%s TLS/HELO %s, failed",cannot,localid);
      mlfi_cleanup(ctx, PASS_MESSAGE);
      return SMFIS_TEMPFAIL;
    }

// Save HELO line in container, free previously allocated (if any)

  sprintf(buf, "%s, %s", helohost, tls);

  if (priv->mlfi_helofrom)
    free(priv->mlfi_helofrom);

  priv->mlfi_helofrom = buf;

// Walk through...

  return SMFIS_CONTINUE;
}

/* ---------------------------------------------------------------------------------
 *
 *	MAIL FROM: catch-up (MAIL FROM: command was arrived)
 *
 *	Call with:	ctx  	 - communications area for sendmail
 *			argv	 - pointer to selected sender's name
 *
 *	Return code:	SMFIS_CONTINUE	- continue message processing
 *			SMFIS_TEMPFAIL	- fatal error was detected, abort message
 *
 * --------------------------------------------------------------------------------- */

sfsistat mlfi_envfrom(SMFICTX *ctx, char **argv)
{
  struct mlfiPriv *priv = MLFIPRIV;
  pid_t curpid;
  time_t curtime;
  char *localid = "(envfrom)";
  struct sccuser *scc1;
  short int i, namelen;

// Clear initial private data structure fields

  priv->mlfi_mailfrom = NULL;
  priv->mlfi_fname = NULL;
  priv->mlfi_fp = priv->mlfi_fpbody = NULL;
  priv->ptm = NULL;
  priv->headerlines = priv->body_parts = 0;
  priv->body_length = priv->recipients = 0;
  *((u_short *) &priv->_cflags) &= 0x4020;
  *((u_short *) &priv->_bflags) = 0;
  priv->work = priv->charset = priv->boundary = NULL;
  priv->charsetlength = priv->boundarylength = 0;

// Allocate memory to temp file name to keep header and all message after check

  namelen = strlen(_SCC_milter_config.isolation) + TEMPLENGTH;

  if ((priv->mlfi_fname = (char *) malloc(namelen + 1)) == NULL)
    {
      syslog(LOG_ERR,"%s header/save temp file name %s, failed",cannot,localid);
      mlfi_cleanup(ctx, PASS_MESSAGE);
      return SMFIS_TEMPFAIL;
    }
#ifdef DBPRINT
   else
     if (_SCC_milter_config.debug_level > 50)
       syslog(LOG_ERR,"%s %s priv->mlfi_fname=0x%X",
       				localid,dblev[1],(int) priv->mlfi_fname);
#endif

// Allocate memory to temp file name to keep message body inside check

  if ((priv->mlfi_bname = (char *) malloc(namelen + 1)) == NULL)
    {
      syslog(LOG_ERR,"%s body temp file name %s, failed",cannot,localid);
      mlfi_cleanup(ctx, PASS_MESSAGE);
      return SMFIS_TEMPFAIL;
    }
#ifdef DBPRINT
   else
     if (_SCC_milter_config.debug_level > 50)
       syslog(LOG_ERR,"%s %s priv->mlfi_bname=0x%X",
       				localid,dblev[1],(int) priv->mlfi_bname);
#endif

// Allocate memory to working space and write additionally pointers (use symbolic
// constants)

  if ((priv->work = (char *) calloc(WORKLENGTH,1)) == NULL)
    {
      syslog(LOG_ERR,"%s workspace %s, failed",cannot,localid);
      mlfi_cleanup(ctx, PASS_MESSAGE);
      return SMFIS_TEMPFAIL;
    }

// Allocate memory to sender data structure and store sender address here
// (when we couldn't find sender in SCC users table, we already will have complete
// users data structure)

  if ((priv->mlfi_mailfrom =
  		(struct sccuser *) calloc(sizeof(struct sccuser),1)) == NULL)
    {
      syslog(LOG_ERR,"%s sender data structure %s, failed",cannot,localid);
      mlfi_cleanup(ctx, PASS_MESSAGE);
      return SMFIS_TEMPFAIL;
    }
   else
     strcpy(priv->mlfi_mailfrom->sccuser,argv[0]);

// Allocate memory to time repository

  if ((priv->ptm = (struct tm *) malloc(sizeof(struct tm))) == NULL)
    {
      syslog(LOG_ERR,"%s time structure %s, failed",cannot,localid);
      mlfi_cleanup(ctx, PASS_MESSAGE);
      return SMFIS_TEMPFAIL;
    }

// Get some values to make a temp file name

  curpid = getpid();
  curtime = time(NULL);

  localtime_r(&curtime,priv->ptm);

// Preparing temp file name for keeping header

  snprintf(priv->mlfi_fname,namelen + 1,
        "%s/Binf_%.2d%.2d%.2d%.2d%.2d%.2d.XXXXXX",_SCC_milter_config.isolation,
        (int) priv->ptm->tm_mday, (int) priv->ptm->tm_mon + 1,
        (int) (priv->ptm->tm_year - 100), (int) priv->ptm->tm_hour,
        (int) priv->ptm->tm_min, (int) priv->ptm->tm_sec);

// Copy it to filename to keep body in analyze time

  strcpy(priv->mlfi_bname, priv->mlfi_fname);

#ifdef DBPRINT
  if (_SCC_milter_config.debug_level > 50)
    syslog(LOG_INFO,"%s %s constructed temp file is %s",
    				localid,dblev[1],priv->mlfi_fname);
#endif

// Open and chmod temp file to keep header

  if (!(_SCC_milter_create_temp_file(priv->mlfi_fname, &priv->mlfi_fp)))
    {
      mlfi_cleanup(ctx, PASS_MESSAGE);
      return SMFIS_TEMPFAIL;
    }
   else
     priv->_cflags._tempsave = OK;

// Open and chmod temp file to keep body

  if (!(_SCC_milter_create_temp_file(priv->mlfi_bname, &priv->mlfi_fpbody)))
    {
      mlfi_cleanup(ctx, PASS_MESSAGE);
      return SMFIS_TEMPFAIL;
    }
   else
     priv->_cflags._tempbody = OK;

// Store connection address and HELO host (if available)

  if (fprintf(priv->mlfi_fp,
        "Message tracing:\n----------------\nConnect from %s (%s)\n",
  	             priv->mlfi_helofrom, priv->mlfi_connectfrom) == EOF)
    {
      mlfi_cleanup(ctx, PASS_MESSAGE);
      return SMFIS_TEMPFAIL;
    }

// Store MAIL FROM: indentation.

  if (fprintf(priv->mlfi_fp, "MAIL FROM: %s\n",argv[0]) == EOF)
    {
      mlfi_cleanup(ctx, PASS_MESSAGE);
      return SMFIS_TEMPFAIL;
    }

#ifdef DBPRINT
// Here we print SCC users table
  if (_SCC_milter_config.debug_level > 50)
    {
      syslog(LOG_ERR,"%s %s SCC users table (all is %d)",localid,dblev[1],qsccusers);

      for(i = 1, scc1 = sccusers;i <= qsccusers;i++,scc1++)
       syslog(LOG_ERR,"%s %s scc->sccuser=%s,scc->_uflags=0x%X,scc->sccpages=%s,scc=0x%X",
   	   localid,dblev[1],scc1->sccuser,*((u_char *) &scc1->_uflags),
   	   						scc1->sccpages,(int) scc1);

    }
#endif

// Walk through SCC users file and check, if sender allowed to take appropriate
// actions. When we found sender description (by address), we copy whole structure
// in a sender data structure

  for(i = 1, scc1 = sccusers;i <= qsccusers;i++,scc1++)
   if ((strcmp(scc1->sccuser,argv[0])) == 0)
     {
       memcpy((char *) priv->mlfi_mailfrom, (char *) scc1, sizeof(struct sccuser));

#ifdef DBPRINT
       if (_SCC_milter_config.debug_level > 20)
         syslog(LOG_ERR,"%s %s Sender match found",localid,dblev[0]);

       if (_SCC_milter_config.debug_level > 50)
         syslog(LOG_ERR,"%s %s mlfi->sccuser=%s,mlfi->_uflags=0x%X,mlfi->sccpages=%s",
   	   	  localid,dblev[1],priv->mlfi_mailfrom->sccuser,
   	   	  	*((u_char *) &scc1->_uflags),priv->mlfi_mailfrom->sccpages);
#endif

       break;
     }

// When we couldn't find sender in SCC users table, we point to empty sccuser structure
// to indicate missed in SCC users table

#ifdef DBPRINT
  if (i > qsccusers)
    {
      if (_SCC_milter_config.debug_level > 20)
        syslog(LOG_ERR,"%s %s Sender %s has not been found in SCC users table",
          						localid,dblev[0],argv[0]);
    }
#endif

// We recognize local domain and set sing about it

  if ((strstr(priv->mlfi_mailfrom->sccuser,_SCC_milter_config.domain)) != 0)
    priv->_cflags._localout = OK;

// Walk through...

  return SMFIS_CONTINUE;
}

/* ---------------------------------------------------------------------------------
 *
 *	RCPT TO: catch-up (RCPT TO: command was arrived)
 *
 *	Call with:	ctx  	 - communications area for sendmail
 *			argv	 - pointer to selected recipient's name
 *
 *	Return code:	SMFIS_CONTINUE	- continue message processing
 *			SMFIS_TEMPFAIL	- fatal error was detected, abort message
 *
 * --------------------------------------------------------------------------------- */

sfsistat mlfi_envrcpt(SMFICTX *ctx, char **argv)
{
  struct mlfiPriv *priv = MLFIPRIV;
  char *localid = "(envrcpt)";
  struct sccuser *scc1,*scc2;
  short int i;

// Allocate RCPT TO: local container and store to it

  if (!priv->_cflags._firstrcpt)
    if ((priv->mlfi_rcptto =
    	  (struct sccuser *) calloc(sizeof(struct sccuser),5)) == NULL)
      {
        syslog(LOG_ERR,"%s recipients address pointers array %s, failed",cannot,localid);
        mlfi_cleanup(ctx, PASS_MESSAGE);
        return SMFIS_TEMPFAIL;
      }
       else
         {
           priv->_cflags._firstrcpt = OK;
#ifdef DBPRINT
           if (_SCC_milter_config.debug_level > 100)
             syslog(LOG_ERR,"%s %s priv->_cflags=0x%X,recipient=%s,priv->mlfi_rcptto=0x%X",
           	 localid,dblev[2],*((u_short *) &priv->_cflags),
           	 			argv[0],(int) priv->mlfi_rcptto);
#endif

         }
   else

// When we reached a 5/10/20 recipients for this letter, we reallocate memory for
// keep recipients parameters (if any).

     if ((priv->recipients == 5) || (priv->recipients == 10) ||
     						(priv->recipients == 20))
       if ((priv->mlfi_rcptto =
       	   (struct sccuser *) realloc(priv->mlfi_rcptto,
       	   		sizeof(struct sccuser) * (priv->recipients * 2))) == NULL)
         {
           syslog(LOG_ERR,"%s realloc recipients address array %s, failed",cannot,localid);
           mlfi_cleanup(ctx, PASS_MESSAGE);
           return SMFIS_TEMPFAIL;
         }

// Walk through SCC users file and check, if recipient allowed to take appropriate
// actions. Preliminary we set position in recipients array to store pointer to
// current recipient

  scc2 = priv->mlfi_rcptto + priv->recipients;

  for(scc1 = sccusers,i = 1;i <= qsccusers;i++,scc1++)
   {
#ifdef DBPRINT
     if (_SCC_milter_config.debug_level > 50)
       syslog(LOG_ERR,"%s %s We search recipients, in mail - %s, in table now %s",
     					     localid,dblev[1],argv[0],scc1->sccuser);
#endif

     if ((strcmp(scc1->sccuser,argv[0])) == 0)
       {
         memcpy((char *) scc2, (char *) scc1, sizeof(struct sccuser));

#ifdef DBPRINT
         if (_SCC_milter_config.debug_level > 20)
            syslog(LOG_ERR,"%s %s Recipient match found",localid,dblev[0]);

         if (_SCC_milter_config.debug_level > 100)
           syslog(LOG_ERR,"%s %s scc1=0x%X,scc2=0x%X,scc2->sccuser=%s,recipient %d",
           		 localid,dblev[2],(int) scc1,(int) scc2,
           		 			scc2->sccuser,priv->recipients);
#endif

         break;

       }
   }

// When we couldn't find current recipient in SCC users table, we point to empty
// sccuser structure to indicate missed in SCC users table. We storing pointer
// to pointer to empty structure and fill in

  if (i > qsccusers)
    {
      strcpy(scc2->sccuser,argv[0]);

#ifdef DBPRINT
      if (_SCC_milter_config.debug_level > 20)
        syslog(LOG_ERR,"%s %s Recipient %s has not been found in SCC users table",
          						   localid,dblev[1],argv[0]);
#endif
    }

// Log all RCPT TO: tries to message tracing part of message dump

  if (fprintf(priv->mlfi_fp, "RCPT TO: %s\n",argv[0]) == EOF)
    {
      mlfi_cleanup(ctx, PASS_MESSAGE);
      return SMFIS_TEMPFAIL;
    }
     else
       priv->recipients++;

// Walk through...

  return SMFIS_CONTINUE;
}

/* ---------------------------------------------------------------------------------
 *
 *	header line catch-up (next header line was arrived)
 *
 *	Call with:	ctx  	 - communications area for sendmail
 *			headerf	 - header field
 *			headerv	 - header value
 *
 *	Return code:	SMFIS_CONTINUE	- continue message processing
 *			SMFIS_DISCARD	- discard this message
 *			SMFIS_REJECT	- reject this message
 *			SMFIS_TEMPFAIL	- fatal error was detected, abort message
 *
 *	Notes: return codes implied from status_check() routine or exactly
 *
 * --------------------------------------------------------------------------------- */

sfsistat mlfi_header(SMFICTX *ctx, char *headerf, char *headerv)
{
  struct mlfiPriv *priv = MLFIPRIV;
  char *localid = "(header)";
  char *pt,*pta;
  sfsistat lout;
  short int charsetlength;

// If it is a first header, close Message tracing

  if (!priv->_cflags._firstheader)
    if (fprintf(priv->mlfi_fp,"----------------\n\n") == EOF)
      {
        syslog(LOG_ERR,"fprintf %s failed: %m",localid);
        mlfi_cleanup(ctx, PASS_MESSAGE);
        return SMFIS_TEMPFAIL;
      }
       else
         priv->_cflags._firstheader = OK;
   else ;

// Print header field and value in message dump

  if (fprintf(priv->mlfi_fp,"%s: %s\n",headerf,headerv) == EOF)
    {
      syslog(LOG_ERR,"fprintf %s failed: %m",localid);
      mlfi_cleanup(ctx, PASS_MESSAGE);
      return SMFIS_TEMPFAIL;
    }

  priv->headerlines++;

// When this headreline is a "Content-Type", do some checks

  if ((strcasecmp(headerf,"Content-Type")) == 0)
    {

#ifdef DBPRINT
      if (_SCC_milter_config.debug_level > 20)
        syslog(LOG_ERR,"%s %s We got: %s - %s",localid,dblev[0],headerf,headerv);
#endif

// We checking Content-Type line against media type and charset (if any). Length of
// Conteny-Type line accounted also trailing newline.

      if ((sccerror =
      	    _SCC_milter_check_content_type(ctx,
      	    		headerv,strlen(headerv) + 1,HEADER)) != NOERROR)
        {

// Check on passed return code - we now have spam codes and enterprise codes

          if (sccerror > ENOERROR)
            priv->_cflags._isenterviol = OK;
           else
             priv->_cflags._isspam = OK;

// When we aliready detected spam, no further check need. This subroutine self sets
// return code, when wasn't imitate mode, we stop checking here.

      	  if ((lout = _SCC_milter_status_check(ctx,sccerror)) != SMFIS_CONTINUE)
            return lout;
        }
    }				// if ... Content-Type

// When we processing Subject field, do some checks...

  if ((strcasecmp(headerf,"Subject")) == 0)
    {

// We have found a codepage srecification in Subject line - we should check in against
// user-specified or global codepage

      if ((pt = (strstr(headerv,"=?"))) != 0)
        {

#ifdef DBPRINT
          if (_SCC_milter_config.debug_level > 20)
            syslog(LOG_ERR,"%s %s Detected charset specification in subject",
        							localid,dblev[0]);
#endif
          pta = strchr(pt + 2,'?');

// When charset memory didn't allocated yet, we do this. Else we clean allocated space
// When cleaning is active (instead allocating), keep in mind - '\0' is MANDATORY!

          charsetlength = pta - pt - 2;

          if (!priv->charset)
            {
              priv->charsetlength = charsetlength;
              priv->charset = (char *) calloc(priv->charsetlength + 1,1);

#ifdef DBPRINT
              if (_SCC_milter_config.debug_level > 20)
                syslog(LOG_ERR,"%s %s Allocating %d bytes for charset",
        				localid,dblev[0],priv->charsetlength + 1);
#endif
            }
           else
             {
               if (charsetlength > priv->charsetlength)
                 {
                   priv->charset = (char *) realloc(priv->charset,charsetlength + 1);
                   priv->charsetlength = charsetlength;

#ifdef DBPRINT
                   if (_SCC_milter_config.debug_level > 100)
                     syslog(LOG_ERR,"%s %s Reallocating %d bytes for charset",
        				 	   localid,dblev[2],charsetlength);
#endif
                 }
                else
                  {
                    priv->charsetlength = charsetlength;
                    bzero(priv->charset,priv->charsetlength + 1);

#ifdef DBPRINT
                    if (_SCC_milter_config.debug_level > 20)
                      syslog(LOG_ERR,"%s %s Cleaning %d bytes in charset field",
        				  localid,dblev[0],priv->charsetlength + 1);
#endif
                  }
             }			// if (priv->charset)

// Move charset description (exclude =? markers) to charset space

          memmove(priv->charset, pt + 2, priv->charsetlength);

          if (_SCC_milter_check_charset(ctx,priv->charset))
            priv->_cflags._isspam = OK;

        }			// if =? was found...
     }				// if "Subject" was found...

// When we aliready detected spam, no further check need. This subroutine self sets
// return code, when wasn't imitate mode, we stop checking here.

  if ((lout = _SCC_milter_status_check(ctx,sccerror)) != SMFIS_CONTINUE)
    return lout;

// Walk through...

  return SMFIS_CONTINUE;
}

/* ---------------------------------------------------------------------------------
 *
 *	end-of-header catch-up (message header was finished, body awaited)
 *
 *	Call with:	ctx  	 - communications area for sendmail
 *
 *	Return code:	SMFIS_CONTINUE	- continue message processing
 *			SMFIS_DISCARD	- discard this message
 *			SMFIS_REJECT	- reject this message
 *			SMFIS_TEMPFAIL	- fatal error was detected, abort message
 *
 *	Notes: return codes implied from status_check() routine or exactly
 *
 * --------------------------------------------------------------------------------- */

sfsistat mlfi_eoh(SMFICTX *ctx)
{
  struct mlfiPriv *priv = MLFIPRIV;
  char *localid = "(eoh)";
  sfsistat lout;

// Put one empty line to separate message headers and body

  if (fprintf(priv->mlfi_fp,"\n") == EOF)
    {
      syslog(LOG_ERR,"fprintf %s failed: %m",localid);
      mlfi_cleanup(ctx, PASS_MESSAGE);
      return SMFIS_TEMPFAIL;
    }

// When it isn't multipart message and we didn't detect charset in Content-Type,
// we take a NoCharsetSpecify action

#ifdef DBPRINT
  if (_SCC_milter_config.debug_level > 50)
    syslog(LOG_ERR,"%s %s priv->_cflags=0x%X",
    			localid,dblev[1],*((u_short *) &priv->_cflags));
#endif

// When this is a not-multipart message and "charset=" hasn't been found, we checking
// if this occassion qualified as spam or no.

  if (!priv->_cflags._multipart)
    {
      if (!priv->_cflags._charset)
        {
          priv->_cflags._isspam = OK;
          sccerror = 4;

#ifdef DBPRINT
          if (_SCC_milter_config.debug_level > 20)
            syslog(LOG_ERR,
            	"%s %s Not multipart and no charset has been found",localid,dblev[0]);
#endif

          if ((lout = _SCC_milter_status_check(ctx,sccerror)) != SMFIS_CONTINUE)
            return lout;
	   else
             {
               priv->_cflags._isspam = NO;

#ifdef DBPRINT
               if (_SCC_milter_config.debug_level > 20)
                 syslog(LOG_ERR,
             	    "%s %s Conform: we assumed default Content-Type and supposed that it is not spam",
             						localid,dblev[0]);
#endif

             }
        }				// if no charset...

    }					// if not multipart...

// Else we check boundary presence, when boundary wasn't specified (it is possible
// in RFC), we set default boundary "--"

   else
     if (!priv->_cflags._boundary)
       {
#ifdef DBPRINT
         if (_SCC_milter_config.debug_level > 20)
	   syslog(LOG_INFO,
	   		"%s %s Boundary not found, drop to default",localid,dblev[0]);

// Here we check memory assignment - when memory already was not allocated (it is a
// violation of logic, but fool protection), we allocate it now.

         if (_SCC_milter_config.debug_level > 100)
	   syslog(LOG_INFO, "%s %s Boundary area pointer is 0x%X",
	   				localid, dblev[3],(int) priv->boundary);
#endif

	 if (priv->boundary == NULL)
           priv->boundary = (char *) calloc(10,1);

// This is a multipart message, but boundary wasn't specifird. This is legal for
// mail, and we should prosessing this occassion - assuming default boundary
// as "--" line with length 2 (and one ZSB char)

         strcpy(priv->boundary,"--");
         priv->boundarylength = 3;
         priv->_cflags._boundary = OK;
       }

// Walk through...

  return SMFIS_CONTINUE;
}

/* ---------------------------------------------------------------------------------
 *
 *	message body catch-up (next body chunk was arrived)
 *
 *	Call with:	ctx  	 - communications area for sendmail
 *			bodyp	 - pointer to body chunk
 *			bodylen  - length of this chunk
 *
 *	Return code:	SMFIS_CONTINUE	- continue message processing
 *			SMFIS_TEMPFAIL	- fatal error was detected, abort message
 *
 *	Notes: return codes implied from status_check() routine or exactly
 *	       Here was message processing, but with separate file it was
 *	       moved to mlfi_eom()
 *
 * --------------------------------------------------------------------------------- */

sfsistat mlfi_body(SMFICTX *ctx, unsigned char *bodyp, size_t bodylen)
{
  struct mlfiPriv *priv = MLFIPRIV;
  char *localid = "(body)";
  int nwritten;

// Store body part (or entire body, when small)

#ifdef DBPRINT
  if (_SCC_milter_config.debug_level > 20)
    syslog(LOG_ERR,"%s %s Body part %d, length is %d",
    					localid,dblev[0],priv->body_parts,bodylen);
#endif

  if ((nwritten = fwrite(bodyp, bodylen, 1, priv->mlfi_fpbody)) != OK)
    {
      syslog(LOG_ERR,"%s [50] fwrite failed: %m", localid);
      mlfi_cleanup(ctx, PASS_MESSAGE);
      return SMFIS_TEMPFAIL;
    }

#ifdef DBPRINT
   else
     if (_SCC_milter_config.debug_level > 20)
       syslog(LOG_ERR, "%s %s %d bytes were written in message body file",
       					localid, dblev[1], bodylen * nwritten);
#endif
// Accumulate quantity of parts and summary length of message body

  priv->body_parts++;
  priv->body_length += bodylen;

// Walk through...

  return SMFIS_CONTINUE;
}

/* ---------------------------------------------------------------------------------
 *
 *	End-of-message catch-up (the message was finished, going to next message)
 *
 *	Call with:	ctx  	 - communications area for sendmail
 *
 *	Return code:	SMFIS_CONTINUE	- continue message processing
 *			SMFIS_DISCARD	- discard this message
 *			SMFIS_REJECT	- reject this message
 *			SMFIS_TEMPFAIL	- fatal error was detected, abort message
 *
 *	Notes: return codes implied from status_check() routine
 *
 * --------------------------------------------------------------------------------- */

sfsistat mlfi_eom(SMFICTX *ctx)
{
  struct mlfiPriv *priv = MLFIPRIV;
  char *localid = "(eom)";
  sfsistat lout;
  char *pta,*ptd,*pte;
  size_t readlength, accumulated;
  int messageparts = 0;

// Rewind message body file to begin

  rewind(priv->mlfi_fpbody);

// When we have received multipart message, we consequently search boundary, and when
// boundary was found, start process from this position as from part begin

  if (priv->_cflags._multipart)
    {

#ifdef DBPRINT
      if (_SCC_milter_config.debug_level > 20)
        syslog(LOG_INFO,
        	"%s %s Multipart message, search boundaries now",localid,dblev[0]);
#endif

// Main loop with reading stored message body data and checking against...

      for(readlength = accumulated = 0;!(feof(priv->mlfi_fpbody));)
       {
         if ((pta = fgetln(priv->mlfi_fpbody, &readlength)) == NULL)
           {
             syslog(LOG_ERR,"%s error reading saved message body: %m",localid);

// This is a debug code to temp save error generating mail
             priv->_cflags._dbgsave = 1;
             break;
//             mlfi_cleanup(ctx, PASS_MESSAGE);
//             return SMFIS_TEMPFAIL;
           }

#ifdef DBPRINT
#warning BEWARE! debug level > 120 generates HUGE amount of debug info!
         if (_SCC_milter_config.debug_level > 120)
           syslog(LOG_ERR,"%s %s readline=0x%X, was read %d bytes",
           				localid,dblev[4],(int) pta, readlength);
#endif

// When "boundary stopper" was already found - no data should be follwed, this is a
// sendmail mistake.

      	 if (priv->_cflags._bstop)
           {
#ifdef DBPRINT
             if (_SCC_milter_config.debug_level > 20)
               syslog(LOG_ERR,"%s %s Boundary stopper was found, none data assumed",
               							    localid,dblev[0]);
#endif

             break;
           }

// We found a boundary, so this place is a start a new message part

         if ((ptd = __strnstr(pta, priv->boundary, readlength)) != NULL)
           {
             accumulated += readlength;
             messageparts++;

#ifdef DBPRINT
             if (_SCC_milter_config.debug_level > 20)
	       syslog(LOG_INFO,"%s %s Adding message parts counter, now is %d",
             					    localid,dblev[0],messageparts);

             if (_SCC_milter_config.debug_level > 20)
	       syslog(LOG_INFO,"%s %s We found a new boundary (%d) at offset %d",
	     		   localid,dblev[0],messageparts,accumulated + (ptd - pta));
#endif

// When we detect at tail of boundary text two '-' chars - this is a "boundary stopper"
// and we MUST stop processing data - no one bounaries will follows

             if (!(strncmp(ptd + strlen(priv->boundary),"--",2)))
               {

#ifdef DBPRINT
                 if (_SCC_milter_config.debug_level > 20)
                   syslog(LOG_ERR,
                   	"%s %s Boundary stopper found, break apart",localid,dblev[0]);
#endif

                 priv->_cflags._bstop = OK;
                 break;
               }

// Drop any part type signes to default, allow to analyze this part

             *((u_short *) &priv->_bflags) &= 0x0000;

// Set a presence new boundary

	     priv->_bflags._incomplete = OK;

             continue;
           }		// if ... boundary...

// When this line is line from MIME part header

         if (priv->_bflags._incomplete)
           {

// When this line is a "\r\n", and this is a second similar line - MIME header was
// finished and body was started next line

             if ((memcmp(pta, "\r\n", readlength)) == NULL)
               {
                 priv->_bflags._incomplete = NO;

#ifdef DBPRINT
                 if (_SCC_milter_config.debug_level > 20)
                   syslog(LOG_ERR, "%s %s MIME header finished", localid, dblev[0]);
#endif
                 continue;
               }

// We found a Content-Type in this boundary

             if ((ptd = __strnstr(pta,"Content-Type",readlength)) != NULL)
               {

// We checking Content-Type line against HTML and charset

#ifdef DBPRINT
                 if (_SCC_milter_config.debug_level > 100)
                   syslog(LOG_ERR,"%s %s Content-Type from 0x%X, offset=%d",
                   	   localid, dblev[2], (int) ptd, accumulated + (ptd - pta));
#endif

// Here we search "Content-Type" value, when missed - it is spam

                 if ((pte = memchr(ptd,' ',readlength - (ptd - pta))) == NULL)
                   {
                     priv->_cflags._isspam = OK;
                     sccerror = INVMSCTYPE;

      	             if ((lout = _SCC_milter_status_check(ctx,sccerror))
      	            						!= SMFIS_CONTINUE)
                       return lout;
                   }

// Pass line to check against Content-Type.

                 if ((sccerror =
                 	_SCC_milter_check_content_type(ctx,
                 	   pte + 1, readlength - (pte - pta), BODY)) != NOERROR)
                   {

// Check on passed return code - we now have spam codes and enterprise codes

                     if (sccerror > ENOERROR)
                       priv->_cflags._isenterviol = OK;
                      else
                        priv->_cflags._isspam = OK;

// When we aliready detected spam, no further check need. This subroutine self sets
// return code, when wasn't imitate mode, we stop checking here.

      	             if ((lout = _SCC_milter_status_check(ctx,sccerror))
      	            						!= SMFIS_CONTINUE)
                       return lout;
                   }

// Mark Content-Type presence

		 priv->_bflags._iscontype = OK;
                 continue;
               }	       // if Content-Type...

// When we take additional line from Content-Type

	     if (priv->_bflags._iscontype)

// When new line starts with space or tab - Content-Type was continued next line

               if ((*pta == ' ') || (*pta == '\t'))

// Pass line to check against Content-Type.

                 if ((sccerror =
                 	_SCC_milter_check_content_type(ctx,
                 	   	pta + 1, readlength - 1, BODY)) != NOERROR)
                   {

// Check on passed return code - we now have spam codes and enterprise codes

                     if (sccerror > ENOERROR)
                       priv->_cflags._isenterviol = OK;
                      else
                        priv->_cflags._isspam = OK;

// When we aliready detected spam, no further check need. This subroutine self sets
// return code, when wasn't imitate mode, we stop checking here.

      	             if ((lout = _SCC_milter_status_check(ctx,sccerror))
      	            						!= SMFIS_CONTINUE)
                       return lout;
                   }
                  else ;
                else
                  priv->_bflags._iscontype = NO;
              else ;
           }		       // if incomplete...
       }		       // for (!feof())...
    }			       // if multipart...

// Close temp/save file to check it later, when this mail shouldn't be shadowed

  if ((priv->_cflags._tempsave) && (!priv->mlfi_mailfrom->_uflags._shadow))
    {
      fclose(priv->mlfi_fp);
      priv->_cflags._tempsave = NO;
    }

// Close temp/body file to check it later

  if ((priv->_cflags._tempbody) && (!priv->mlfi_mailfrom->_uflags._shadow))
    {
      fclose(priv->mlfi_fpbody);
      priv->_cflags._tempbody = NO;
    }

// When it is multipart message and we didn't detect charset in Content-Type,
// we take a NoCharsetSpecify action

  if (priv->_cflags._multipart)
    if (!priv->_cflags._charset)
      {
        if (_SCC_milter_config._idflags._nocharset)
          {
            priv->_cflags._isspam = OK;
            sccerror = 4;
          }

        if ((lout = _SCC_milter_status_check(ctx,sccerror)) != SMFIS_CONTINUE)
          return lout;
      }

// When we here, it means, this message is not spam

#ifdef DBPRINT
  if (_SCC_milter_config.debug_level > 20)
    syslog(LOG_INFO,
    	"%s %s This message is normal, cflags=0x%X,bflags=0x%X",
  			localid,dblev[0],*((u_short *) &priv->_cflags),
  						*((u_short *) &priv->_bflags));
#endif

  smfi_addheader(ctx,"X-Spam-Control","Normal message");

// Recite message, when requested

  if (!priv->mlfi_mailfrom->_uflags._shadow)
    mlfi_cleanup(ctx,PASS_MESSAGE);
   else
     mlfi_cleanup(ctx,RECITE_MESSAGE);

// Walk through...

  return SMFIS_CONTINUE;
}

/* ---------------------------------------------------------------------------------
 *
 *	Abort mail session (some error was detected)
 *
 *	Call with:	ctx  	 - communications area for sendmail
 *
 *	Return code:	SMFIS_CONTINUE	- continue message processing
 *
 * --------------------------------------------------------------------------------- */

sfsistat mlfi_abort(SMFICTX *ctx)
{
  return mlfi_cleanup(ctx, PASS_MESSAGE);
}

/* ---------------------------------------------------------------------------------
 *
 *	Clean-up mail session (one letter was transmitted, accepted or declined,
 *			       going to next letter. Connection didn't terminated,
 *			       so, connection-oriented parameters (mlfi_connect,
 *			       mlfi_helo and priv itself CANNOT free!)
 *
 *	Call with:	ctx  	 - communications area for sendmail
 *			recite	 - should we recite this message or pass it
 *
 *	Return code:	SMFIS_CONTINUE	- continue message processing
 *
 *	Notes: Messages reciting will be done in enterprise environment to save
 *	       some or all messages, going to some or all people (this feature
 *	       will useful for security deps, usually presented for military
 *	       factories, researchs etc. in Russia)
 *
 * --------------------------------------------------------------------------------- */

sfsistat mlfi_cleanup(SMFICTX *ctx, char recite)
{
  struct mlfiPriv *priv = MLFIPRIV;
  char dflag = 0;
  char *localid="(cleanup)";

// Sanity check (patch provided by Damir Bikmuhametov)

  if (!priv)
    return SMFIS_CONTINUE;

// Recite message, when requested

  if (recite == RECITE_MESSAGE)
    {

#ifdef DBPRINT
      if (_SCC_milter_config.debug_level > 20)
        syslog(LOG_INFO,"%s %s Message from %s to %s will be recited",localid,dblev[0],
    			    priv->mlfi_mailfrom->sccuser,priv->mlfi_rcptto->sccuser);
#endif

      _SCC_milter_recite_message(ctx);
    }

// Free MAIL FROM: repository

  if (priv->mlfi_mailfrom)
    {
      free(priv->mlfi_mailfrom);
      priv->mlfi_mailfrom = NULL;
    }

// Free RCPT TO: repository

  if (priv->mlfi_rcptto)
    {
      free(priv->mlfi_rcptto);
      priv->mlfi_rcptto = NULL;
    }

  priv->recipients = 0;

// Free allocated time repository

  if (priv->ptm)
    {
      free(priv->ptm);
      priv->ptm = NULL;
    }

// Close opened temporary files (file for message body should be closed and deleted
// this time, but when message was recited, these FCB's still exist)

  if (priv->_cflags._tempsave)
    {
      fclose(priv->mlfi_fp);
      priv->mlfi_fp = NULL;
      priv->_cflags._tempsave = NO;
    }

  if (priv->_cflags._tempbody)
    {
      fclose(priv->mlfi_fpbody);
      priv->mlfi_fpbody = NULL;
      priv->_cflags._tempbody = NO;
    }

// Delete file and free allocated filename space. File will deleted only: it contailned
// non-spam mail and debug < 100, it contained spam mail and --no-delete not set
// Also file can be saved, when error (eom) [50] was deteckted and file will saved
// for future investigate. Also mail will save, when mode is 'isolate' (this will
// reach when _discard and _reject flags set simultaneously)

  if (priv->mlfi_fname != NULL)
    {

#ifdef DBPRINT
      if (_SCC_milter_config.debug_level > 50)
        syslog(LOG_INFO,
    	    "%s %s Message flags are: cflags=0x%X,bflags=0x%X",
  			localid,dblev[0],*((u_short *) &priv->_cflags),
  						*((u_short *) &priv->_bflags));

      if (_SCC_milter_config.debug_level > 100)	// When debug > 100 - keep
        {
          if (_SCC_milter_config.debug_level > 50)   	   // Warn about keeping message
	    syslog(LOG_ERR,"%s %s Mail keeping about debug level",localid,dblev[1]);

          dflag = 1;
        }
#endif

      if (_flags._nodelete)                           // Any mail with --no-delete
        {
          dflag = 1;

#ifdef DBPRINT
          if (_SCC_milter_config.debug_level > 50)   	   // Warn about keeping message
	    syslog(LOG_ERR,"%s %s Global mail keeping flag set",localid,dblev[1]);
#endif

        }

      if (((priv->_cflags._isspam) || (priv->_cflags._isenterviol)) &&
              						(_flags._spam_nodelete))
        {
          dflag = 1;                              // Spam mail with --spam-no-delete

#ifdef DBPRINT
          if (_SCC_milter_config.debug_level > 50)   	   // Warn about keeping message
	    syslog(LOG_ERR,"%s %s Global SPAM mail keeping flag set",localid,dblev[1]);
#endif

        }

      if (priv->_cflags._dbgsave)
        {
          dflag = 1;

#ifdef DBPRINT
          if (_SCC_milter_config.debug_level > 50)   	   // Warn about keeping message
	    syslog(LOG_ERR,"%s %s Save mail to future investigation",localid,dblev[1]);
#endif

        }

      if (((priv->_cflags._isspam) && (_SCC_milter_config.mode._isolate)) ||
           ((priv->_cflags._isenterviol) && (_SCC_milter_config.emode._isolate)))
        {
          dflag = 1;

#ifdef DBPRINT
          if (_SCC_milter_config.debug_level > 50)   	   // Warn about keeping message
	    syslog(LOG_ERR,
	    	"%s %s Save mail about isolate mode active",localid,dblev[1]);
#endif

        }

// Delete temp file, when not requested opposite

      if (!dflag)
        {
          unlink(priv->mlfi_fname);
          unlink(priv->mlfi_bname);
        }

      free(priv->mlfi_fname);
      priv->mlfi_fname = NULL;

      free(priv->mlfi_bname);
      priv->mlfi_bname = NULL;
    }

// Clearing per-message counters

  bzero(priv->work,WORKLENGTH);

  priv->headerlines = 0;
  priv->body_parts = 0;
  priv->body_length = 0;
  *((u_short *) &priv->_cflags) &= 0x0020;
  *((u_short *) &priv->_bflags) = 0;
  priv->charsetlength = priv->boundarylength = 0;

// Free allocated charset repository

  if (priv->charset)
    {
      free(priv->charset);
      priv->charset = NULL;
    }

// Free allocated boundary repository

  if (priv->boundary)
    {
      free(priv->boundary);
      priv->boundary = NULL;
    }

// Walk through...

  return SMFIS_CONTINUE;
}

/* ---------------------------------------------------------------------------------
 *
 *	Close mail session (all mail was transmitted, accepted or declined)
 *
 *	Call with:	ctx  	 - communications area for sendmail
 *
 *	Return code:	SMFIS_CONTINUE	- continue message processing
 *
 * --------------------------------------------------------------------------------- */

sfsistat mlfi_close(SMFICTX *ctx)
{
  struct mlfiPriv *priv = MLFIPRIV;

// Sanity check (patch provided by Damir Bikmuhametov)

  if (!priv)
    return SMFIS_CONTINUE;

// Free CONNECT From: repository

  if (priv->mlfi_connectfrom)
    {
      free(priv->mlfi_connectfrom);
      priv->mlfi_connectfrom = NULL;
    }

// Free HELO FROM: repository

  if (priv->mlfi_helofrom)
    {
      free(priv->mlfi_helofrom);
      priv->mlfi_helofrom = NULL;
    }

// Free work space

  if (priv->work)
    {
      free(priv->work);
      priv->work = NULL;
    }

// At last, free private storage itself

  free(priv);
  smfi_setpriv(ctx, NULL);

// Walk through...

  return SMFIS_CONTINUE;
}

/* ---------------------------------------------------------------------------------
 *
 *	Recite message (mail session about to finish, reciting message was
 *			selected for this user, unsuccesful rectiting does not
 *			fatal and does not stop preceeding next message)
 *
 *	Call with:	ctx  	 - communications area for sendmail
 *
 *	Return code:	NO       - cannot recite message, proceeding next
 *			OK	 - message recited
 *
 * --------------------------------------------------------------------------------- */

int _SCC_milter_recite_message(SMFICTX *ctx)
{
  char *maxpath;
  struct mlfiPriv *priv = MLFIPRIV;
  char *localid="(recite)";
  int pathlen;
  FILE *appfile;

// Check on presence directory for reciting and correct permissions

  if (!(_SCC_milter_check_directory(_SCC_milter_config.extrasave)))
    {
      mlfi_cleanup(ctx, PASS_MESSAGE);
      return NO;
    }

// Take memory for temp buffer to make pathway

  if ((maxpath = calloc(MAXPATHLEN * 2, 1)) == NULL)
    {

#ifdef DBPRINT
      if (_SCC_milter_config.debug_level > 20)
	syslog(LOG_ERR,
	    	"%s %s Cannot allocate memory for pathway buffer",localid,dblev[0]);
#endif

      mlfi_cleanup(ctx, PASS_MESSAGE);
      return NO;
    }
   else
     {

// Build pathway as "extrasave + / + username from MAIL FROM: (without domain part)
// and strip '<' char at start

       strcpy(maxpath,_SCC_milter_config.extrasave);
       *(maxpath + strlen(maxpath)) = '/';
       strncat(maxpath,priv->mlfi_mailfrom->sccuser + 1,
       	   strchr(priv->mlfi_mailfrom->sccuser,'@') - priv->mlfi_mailfrom->sccuser - 1);

       pathlen = strlen(maxpath);

#ifdef DBPRINT
       if (_SCC_milter_config.debug_level > 50)
         {
	   syslog(LOG_ERR,"%s %s Built pathway is %s",localid,dblev[1],maxpath);
           syslog(LOG_ERR,"%s %s Pathway length is %d",localid,dblev[1],pathlen);
         }
#endif

     }

// Preparing temp file name for keeping header

  snprintf(maxpath + pathlen,WORKLENGTH,
        "/Binf_%.2d%.2d%.2d%.2d%.2d%.2d.XXXXXX",
        (int) priv->ptm->tm_mday, (int) priv->ptm->tm_mon + 1,
        (int) (priv->ptm->tm_year - 100), (int) priv->ptm->tm_hour,
        (int) priv->ptm->tm_min, (int) priv->ptm->tm_sec);

// Open and chmod temp file to keep body

  if (!(_SCC_milter_create_temp_file(maxpath, &appfile)))
    {
      mlfi_cleanup(ctx, PASS_MESSAGE);
      return SMFIS_TEMPFAIL;
    }

#ifdef DBPRINT
      if (_SCC_milter_config.debug_level > 50)
        syslog(LOG_INFO,
    	    "%s %s Message flags are: cflags=0x%X,bflags=0x%X",
  			localid,dblev[0],*((u_short *) &priv->_cflags),
  						*((u_short *) &priv->_bflags));
#endif

// Copy mail header to extrasave dir

  if (priv->_cflags._tempsave)
    {

#ifdef DBPRINT
      if (_SCC_milter_config.debug_level > 50)
        syslog(LOG_ERR,"%s %s Reciting mail header from fildes %d",
          				localid, dblev[1], fileno(priv->mlfi_fp));
#endif

      fbbcopy(priv->mlfi_fp, appfile, 0600);
    }

// Copy mail body to extrasave dir

  if (priv->_cflags._tempbody)
    {

#ifdef DBPRINT
      if (_SCC_milter_config.debug_level > 50)
        syslog(LOG_ERR,"%s %s Reciting mail body from fildes %d",
          			localid, dblev[1], fileno(priv->mlfi_fpbody));
#endif

      fbbacopy(priv->mlfi_fpbody, appfile, 0600);
    }

// And go away...

  free(maxpath);
  return OK;
}
/* ---------------------------------------------------------------------------------
 *
 *	Check mail status and take appropriate actions
 *
 *	Call with:	ctx  	 - communications area for sendmail
 *			sccerror - SCC error code (for debug and logging)
 *
 *	Return code:	SMFIS_CONTINUE	- continue message processing
 *			SMFIS_DISCARD	- discard this message
 *			SMFIS_REJECT	- reject this message
 *
 *	Notes:		SCC enterprise error codes have error code + 500
 *
 * --------------------------------------------------------------------------------- */

sfsistat _SCC_milter_status_check(SMFICTX *ctx, short int sccerror)
{
  struct mlfiPriv *priv = MLFIPRIV;

  char *spam = "SPAM";
  char *enterprise = "ENTERPRISE";
  char *selector,*terror,*mode,*emode,*amode;

  char bmode = 0;			// Binary mode: 0 - spam, 1 - enterprise

  char *errs[] = { "No errors",
  		   "No permissions to send HTML from this sender",
	           "This charset does not allowed for sender",
	           "This charset does not allowed for recipient",
	           "Badly formed Content-Type or missed charset with text media type",
	           "No permissions to receive HTML for this recipient" };

  char *enerrs[] = {"No errors",
  		    "Encrypted mail does not allowed for this sender",
                    "Executable attachments does not allowed for this sender",
                    "Attachments does not allowed for this sender" };

#ifdef DBPRINT
  char *localid = "(status)";
  struct sccuser *scc1 = priv->mlfi_rcptto;
#endif

// Checking status will done, when _isspam is set and sccerror < 500 or _isenterviol
// is set and sccerror >= 500

  if (((priv->_cflags._isspam) && (sccerror < 500)) ||
  	((priv->_cflags._isenterviol) && (sccerror >= 500)))
    {

// First, we checking error code, and when it is "mailformed Content-Type" and
// we should pass these messages, we pass it :-). Keep in mind - "mailformed
// Content-Type" it is broken boundary, broken main format, but NOT invalid
// or not allowed charset!

#ifdef DBPRINT
      if (_SCC_milter_config.debug_level > 50)
        syslog(LOG_ERR,"%s %s idflags=0x%X",localid,dblev[1],
   				*((u_char *) &_SCC_milter_config._idflags));
#endif

      if ((sccerror == INVMSCTYPE) && (!_SCC_milter_config._idflags._nocharset))
        {

#ifdef DBPRINT
          if (_SCC_milter_config.debug_level > 20)
            syslog(LOG_ERR,
               "%s %s Conforming: we assumed default Content-Type for this message",
                						   localid,dblev[0]);
#endif

          return SMFIS_CONTINUE;
        }

// Detect, if this error code belonged to enterprise section and set correspond
// constants for making right debug messages. Prepare some constants to print message.
// We check sccerror: >= 500 indicated, that we now checking enterprice mode error,
// else spam mode.

      emode = act_modes[*((char *) &_SCC_milter_config.emode)];
      mode = act_modes[*((char *) &_SCC_milter_config.mode)];

      if (sccerror >= 500)
        {
          terror = enerrs[sccerror - 500];
          selector = enterprise;
          amode = emode;
          bmode = 1;
        }
       else
         {
           terror = errs[sccerror];
           selector = spam;
           amode = mode;
         }

// Print info about actions, which we will taking.

#ifdef DBPRINT

// First, we describing called error code

      if (_SCC_milter_config.debug_level > 50)
        syslog(LOG_ERR,"%s %s Status checker was called with error: %s",
        						localid,dblev[1],terror);

      if (_SCC_milter_config.debug_level > 20)
        {

// When this mesage was initiated by local sender, we notify him always

          if (priv->_cflags._localout)
            syslog(LOG_ERR,
            	"%s %s Current actions is: REJECT (local sender)",localid,dblev[0]);
           else
             syslog(LOG_ERR,"%s %s Current actions is: spam - %s, enterprise - %s",
          						 localid,dblev[0],mode,emode);

// Diagnose event, when debug is active

          if (!bmode)
            syslog(LOG_INFO,"%s %s Message from %s to %s qualified as spam",
        	  	localid,dblev[0],priv->mlfi_mailfrom->sccuser, scc1->sccuser);
           else
             syslog(LOG_INFO,"%s %s Message from %s to %s violates an enterprise rules",
        	  	localid,dblev[0],priv->mlfi_mailfrom->sccuser, scc1->sccuser);

          syslog(LOG_INFO,"%s %s Action: %s. Reason: %s",
          					localid,dblev[0],amode,terror);
        }
#endif

// Here we print message about doing spam action. We need this to further batch
// processing to keep statistic about spam/enterprise violations. When local sender
// was initiator by mail transmission, we keep mode REJECT to always notify him
// about errors and appropriatelu changing error message

      if (priv->_cflags._localout)
        syslog(LOG_ERR,"[%s] Mail from %s rejected (local sender) - %s",
      				selector,priv->mlfi_mailfrom->sccuser,terror);
       else
         syslog(LOG_ERR,"[%s] Mail from %s %s - %s",
      			selector,priv->mlfi_mailfrom->sccuser,amode,terror);

      if (_flags._imitate)
        {
#ifdef DBPRINT
          if (_SCC_milter_config.debug_level > 20)
            syslog(LOG_INFO,"%s %s Imitation mode ACTIVE, none action will be issued",
            							      localid,dblev[0]);
#endif

          return SMFIS_CONTINUE;
        }

// We really acted as specified only when processing mail from far outside, when we
// processing mail from inside (recognized LccalDomain) we ALWAYS doing REJECT
// (for avoiding circumstance, when user has sent mail, this was silently discarded
// user think all OK, mail dismissed...:-)
// DISCARD will issued only, when discard fialg set with appropriate type processing

      if ((((_SCC_milter_config.mode._discard) &&
      			(priv->_cflags._isspam) && (!bmode)) ||
      	    ((_SCC_milter_config.emode._discard) &&
      	    		(priv->_cflags._isenterviol) && (bmode)))
      			  			&& (!priv->_cflags._localout))
        {
          mlfi_cleanup(ctx, PASS_MESSAGE);
          return SMFIS_DISCARD;
        }
       else
         {
           smfi_addheader(ctx,"X-Spam-Control",errs[sccerror]);
           smfi_setreply(ctx,SMTPREJECT,SMTPADDREJECT,errs[sccerror]);
           mlfi_cleanup(ctx, PASS_MESSAGE);
           return SMFIS_REJECT;
         }
    }

// Walk through...

  return SMFIS_CONTINUE;
}

/* ---------------------------------------------------------------------------------
 *
 *	Check mail charset against sender and all recipients valid charsets
 *
 *	Call with:	ctx  	- communications area for sendmail
 *			charset - mail charset
 *
 *	Return code:	0    -  charset has been found in list of valid charsets
 *		        1    -  charset hasn't been found in list of valid charsets
 *
 * --------------------------------------------------------------------------------- */

int _SCC_milter_check_charset(SMFICTX *ctx,char *charset)
{
  struct mlfiPriv *priv = MLFIPRIV;
  char *strcp,**strp,*wwork = priv->work;
  struct sccuser *scc1;
  short int i,ssign = 0;

#ifdef DBPRINT
  char *localid = "(checkcharset)";
  char *locsend = "[sender]";
  char *locrecv = "[recipient]";

// Here we will dump recipients table

  if (_SCC_milter_config.debug_level > 50)
    {
      syslog(LOG_ERR,"%s %s charset = %s",localid,dblev[1],charset);
      syslog(LOG_ERR,"%s %s %s Recipients table (all is %d recipients)",
      					localid,locrecv,dblev[1],priv->recipients);

      for(i = 1, scc1 = priv->mlfi_rcptto;i <= priv->recipients;i++,scc1++)
       syslog(LOG_ERR,"%s %s %s scc->sccuser=%s,scc->_uflags=0x%X,scc->sccpages=%s",
   		localid,locrecv,dblev[1],scc1->sccuser,
   				*((u_char *) &scc1->_uflags),scc1->sccpages);
    }
#endif

// When user has a definite charset(s), we search message charset in these. When user
// hasn't definite charset, we search it with global AllowedCharset parameter

  if (priv->mlfi_mailfrom->_uflags._ischarset)
    strcpy(priv->work,priv->mlfi_mailfrom->sccpages);
   else
     strcpy(priv->work,_SCC_milter_config.codepages);

#ifdef DBPRINT
  if (_SCC_milter_config.debug_level > 20)
    syslog(LOG_ERR,"%s %s %s Mail charset is %s, our charset(s) is %s",
    					localid,locsend,dblev[0],charset,priv->work);
#endif

// We check sender charset against user-specific or global charset. When sender charset
// is a '*', all charsets are alliwed. We compare mail charset against each charset
// from charset line (global or user-specific). When we reach eon of line, it means
// we does not found charset, it means this charset not allowed for sender. It is NOT
// critical, probably recipient has allowance to receive mail with this charset

  if (*priv->work != '*')
    {
      for(strp = &wwork;((strcp = strsep(strp,",")) != 0);)
       {
         ssign = NOERROR;

#ifdef DBPRINT
         if (_SCC_milter_config.debug_level > 50)
           syslog(LOG_ERR,"%s %s %s Mail charset is %s, checked charset is %s",
         				    localid,locsend,dblev[1],charset,strcp);
#endif

         if ((strcasecmp(charset,strcp)) == 0)
           {
#ifdef DBPRINT
             if (_SCC_milter_config.debug_level > 20)
               syslog(LOG_ERR,"%s %s %s We found charset %s in allowed line for sender",
               					localid,locsend,dblev[0],charset);
#endif
             break;
           }

         ssign = NOSENDCHARSET;
       }

// When we set sign about non-comparable charsets, we immediately stop further checking
// and goto processing return code. When we scan sender codepage, and it doesn't fit
// and none recipients have personal set codepages, this is spam. When sender's code-
// page doesn't fit, but at least one recipient have personal set codepage, we try
// to check recipients

      if (ssign)
        {
#ifdef DBPRINT
          if (_SCC_milter_config.debug_level > 20)
            syslog(LOG_ERR,
              "%s %s Sender charset did not match, search personal charsets for receivers",
         				    			localid,dblev[0]);
#endif
          for(i = 1,  scc1 = priv->mlfi_rcptto;
          				i <= priv->recipients;i++,scc1++)
           {
             if (scc1->_uflags._ischarset)	// Personal charset presented
               {
#ifdef DBPRINT
                 if (_SCC_milter_config.debug_level > 20)
                   syslog(LOG_ERR,
              	      "%s %s Personal charset(s) presented for receiver %s ",
         				    	localid,dblev[0],scc1->sccuser);
#endif
                 break;
               }
           }

          if (i > priv->recipients)
            {
#ifdef DBPRINT
              if (_SCC_milter_config.debug_level > 20)
                syslog(LOG_ERR,
              	  "%s %s No personal charset(s) found for all receivers, returning %d",
         				    		        localid,dblev[0],ssign);
#endif
              sccerror = ssign;		// Sender charset didn't match and no receivers
              return ssign;
            }
        }   				// if ssign...
       else
         {
#ifdef DBPRINT
           if (_SCC_milter_config.debug_level > 20)
             syslog(LOG_ERR,
               "%s %s Sender charset match, returning %d",localid,dblev[0],NOERROR);
#endif
           return NOERROR;		// Sender charset matched
         }
    }					// if priv->work != '*'...

// Check all recipients against user-specifig or global charset. When recipient charset
// is a '*', all charsets are allowed. We compare mail charset against each charset
// from charset line (global or user-specific). When we reach eon of line, it means
// we cannot found mail charset, it means recipient charset not allowed, it means this
// mail is spam. When we found charset in recipient line, even sender hasn't this charset
// as allowed, we arrive this message.

  for(i = 1, scc1 = priv->mlfi_rcptto;i <= priv->recipients;i++,scc1++)
   {
     if (scc1->_uflags._ischarset)
       strcpy(priv->work,scc1->sccpages);
      else
        strcpy(priv->work,_SCC_milter_config.codepages);

     if (*priv->work != '*')
       {
         wwork = priv->work;
#ifdef DBPRINT
         if (_SCC_milter_config.debug_level > 20)
           syslog(LOG_ERR,"%s %s %s Mail charset is %s, our charset(s) is %s",
    					  localid,locrecv,dblev[0],charset,priv->work);
#endif
         for(strp = &wwork;(strcp = strsep(strp,",")) != 0;)
          {
            ssign = NOERROR;

#ifdef DBPRINT
            if (_SCC_milter_config.debug_level > 50)
              syslog(LOG_ERR,"%s %s %s Mail charset is %s, checked charset is %s",
            					localid,locrecv,dblev[1],charset,strcp);
#endif

            if ((strcasecmp(charset,strcp)) == 0)
              {
#ifdef DBPRINT
                if (_SCC_milter_config.debug_level > 20)
                  syslog(LOG_ERR,
              		"%s %s %s We found charset %s in allowed line for recipient",
              					     localid,locrecv,dblev[0],charset);
#endif
                break;
              }

            ssign = NORECVCHARSET;
          }

// When we set sign about non-comparable charsets, we immediately stop further checking
// and goto processing return code

         if (ssign)
           {
             sccerror = ssign;
             return ssign;
           }
       }			// if not *...
   }				// for all recipients

  return ssign;
}

/* ---------------------------------------------------------------------------------
 *
 *	Check string Content-Type against RFC specification (RFC 2045-2049)
 *
 *	Call with:	ctx  	- communications area for sendmail
 *			line 	- line with Content-Type specification
 *			bodylen - maximum length of message part
 *			where	- where call was issued (header or body)
 *
 *	Return code:	0    -  Content-Type: checked, HTML (if any) allowed, charset
 *		       		allowed for sender and all recipients
 *			!= 0 -  Content-Type checked, errors were found,
 *				return is error code
 *
 * --------------------------------------------------------------------------------- */

int _SCC_milter_check_content_type(SMFICTX *ctx,char *line,size_t bodylen,char where)
{
  struct mlfiPriv *priv = MLFIPRIV;
  struct sccuser *scc1;
  short int i,hstat,estat,sstat,speclen;
  char *pt,*ptc,*ptsave,*ctmajor;

#ifdef DBPRINT
  char *localid = "(checkcontent)";
  char *dspsave;

  if (_SCC_milter_config.debug_level > 20)
    {
      syslog(LOG_ERR,"%s %s We got line with %d chars",localid,dblev[0],bodylen);
      syslog(LOG_ERR,"%s %s cflags=0x%X,bflags=0x%X",
      			localid,dblev[0],*((u_short *) &priv->_cflags),
      						*((u_short *) &priv->_bflags));
    }
#endif

  hstat = estat = sstat = 0;

// We search first line separator, and does not check any after it. Content-Type
// header can be in form: "desc\n desc\n desc", so, when after newline doesn't follow
// space, it means, we found end-of-spec.

  for(pt = ptc = line;(ptc - line) < bodylen;pt = ptc + 1)
   {
     if ((ptc = memchr(pt,'\n',bodylen - (pt - line))) == NULL)
       {
          ptc = line + bodylen;
          break;
       }

#ifdef DBPRINT
     if (_SCC_milter_config.debug_level > 100)
       syslog(LOG_ERR,"%s %s pt=0x%X, ptc=0x%X,strlen=%d,bodylen=%d",
       			localid,dblev[2],(int) pt,(int) ptc,(ptc - pt),bodylen);
#endif

     if ((*(ptc + 1) != ' ') && (*(ptc + 1) != '\t'))
       break;
   }

// Calculating Content-Type specification length

  speclen = ptc - line;

#ifdef DBPRINT
  dspsave = calloc(1,speclen + 10);
  memcpy(dspsave,line,speclen);
  strunescape(dspsave,'\r');

  if (_SCC_milter_config.debug_level > 50)
    syslog(LOG_ERR,"%s %s End-of-spec found at 0x%X, length is %d, spec: %s",
    				localid,dblev[1],(int) ptc,strlen(dspsave),dspsave);
#endif

// When we call check_content() first time and hasn't find ';' - this is really
// "media type only". Elsewhere this is next line in Content-Type and can be without
// Whole message also can be wihout parameters (default boundary, default charset)

  if (!priv->_bflags._iscontype)
    {

// First, we separate media type and mark appropriate flag. When we couldn't find
// parameters separator, this means only type/subtype were specified

      if ((ptsave = memchr(line,';',speclen)) == NULL)
        {
          ptsave = ptc;

          if (where == HEADER)
            priv->_cflags._noctparam = OK;
           else
             priv->_bflags._noctparam = OK;

#ifdef DBPRINT
          if (_SCC_milter_config.debug_level > 20)
            syslog(LOG_ERR,
           	"%s %s Presented only media type, cflags=0x%X,bflags=0x%X",
        			localid,dblev[0],*((u_short *) &priv->_cflags),
        					   *((u_short *) &priv->_bflags));
#endif
        }
    }
   else
     ptsave = line - 1;

// Second find type/subtype separator and compute type length. When media type is
// mailformed header, act as InvalidContentType specified. Do this only when media
// type wasn't marked as "parsed"

  if (!priv->_bflags._media)
    {
      if ((ctmajor = memchr(line,'/',(ptsave - line))) == NULL)
        {
#ifdef DBPRINT
          free(dspsave);
#endif
          return INVMSCTYPE;
        }

      i = ctmajor - line;

#ifdef DBPRINT
      if (_SCC_milter_config.debug_level > 100)
        syslog(LOG_ERR,"%s %s ctmajor=0x%X,line=0x%X,ptsave=0x%X,i=%d",
                	 localid,dblev[2],(int) ctmajor,(int) line,(int) ptsave,i);
#endif

// Set appropriate flag. Default type is text (accorded by RFC 2046)

      if ((strncasecmp(line,"image",i)) == NULL)
        priv->_bflags._image = OK;
       else
         if ((strncasecmp(line,"audio",i)) == NULL)
           priv->_bflags._audio = OK;
          else
            if ((strncasecmp(line,"video",i)) == NULL)
              priv->_bflags._video = OK;
             else
               if ((strncasecmp(line,"application",i)) == NULL)
                 priv->_bflags._application = OK;
                else
                  if ((strncasecmp(line,"multipart",i)) == NULL)
                    {
// Multipart can be detected as in whole message (in header), as in part of message
// some boundary)
                      if (where == HEADER)
                        {
                          priv->_cflags._multipart = OK;
#ifdef DBPRINT
                          if (_SCC_milter_config.debug_level > 20)
                            syslog(LOG_ERR,
                        	"%s %s This is a multipart message",localid,dblev[0]);
#endif
                        }
                       else
                         {
                           priv->_bflags._multipart = OK;
#ifdef DBPRINT
                           if (_SCC_milter_config.debug_level > 20)
                             syslog(LOG_ERR,
                        	"%s %s This is a multipart boundary",localid,dblev[0]);
#endif
                         }
                    }
                   else
                     priv->_bflags._text = OK;

// Mark media type parsing

      priv->_bflags._media = OK;

// Check if this is a onepart HTML message. This checking will be done only when
// media type is text.

      if (priv->_bflags._text)
        if ((strncasestr(ctmajor + 1,"html",ptsave - ctmajor - 1)) != NULL)
          {
#ifdef DBPRINT
            if (_SCC_milter_config.debug_level > 50)
              syslog(LOG_ERR,
      	 	"%s %s This is a HTML message,cflags=0x%X,bflags=0x%X",
      	 	 	localid,dblev[1],*((u_short *) &priv->_cflags),
      	 	 				*((u_short *) &priv->_bflags));
#endif

// When 'IsolateHTML' mode active, we check permissions to send/receive HTML message
// accorded to granted permission: direction (sending or receiving (assumed to local
// domain) detected by _localout sign. When receiving, we check: first global flag
// (--nohtml outgoing or --nohtml full), than server (IP) permission, than user
// permission (htmlin). When one of these permissions set, mail granted, else declined
// and selected action will be performed (for local user - ALWAYS will reject!).

            if ((_SCC_milter_config.hmode._incoming)
      	    		|| (_SCC_milter_config.hmode._outgoing))
              {

// We receiving mail from inside. Checking permissions as: global, user
// All underlying can revert up level (user can allow receive when global decline)

                if (priv->_cflags._localout)		       // Local initiator for mail
                  if (_SCC_milter_config.hmode._outgoing)      // Global permissions flag
                    if (priv->mlfi_mailfrom->_uflags._htmlout) // User allowed to send
                      {
                        hstat = 1;           // and set appropriate flag

#ifdef DBPRINT
                        if (_SCC_milter_config.debug_level > 20)
                          syslog(LOG_ERR,"%s %s Local user %s allowed to send HTML",
                   			localid,dblev[0],priv->mlfi_mailfrom->sccuser);
#endif
                      }
                     else
                       hstat = 0; 	// Not allowed sender, we try to search RCPT
                   else
                     {
                       hstat = 1;       // Global permission flag to send HTML was set

#ifdef DBPRINT
                       if (_SCC_milter_config.debug_level > 20)
                         syslog(LOG_ERR,
                   	    "%s %s Global permissions to send HTML",localid,dblev[0]);
#endif
                     }
                 else  			// Non-local initiator for mail (outside)

// We receiving mail from far outside. Checking permissions as: global, server, user
// All underlying can revert up level (server can allow receive when global decline,
// user can allow receive when server decline)

                   if (_SCC_milter_config.hmode._incoming)    // Global permissions flag
                     if (!priv->_cflags._htmlfrom)		     // Server declined to receive
                       if (priv->mlfi_mailfrom->_uflags._htmlin) // User allowed to send
                         {
                           hstat = 1;		// and set appropriate flag

#ifdef DBPRINT
                           if (_SCC_milter_config.debug_level > 20)
                             syslog(LOG_ERR,"%s %s Remote user %s allowed to send HTML",
                   			localid,dblev[0],priv->mlfi_mailfrom->sccuser);
#endif
                         }
                        else
                          hstat = 0;  		// User not allowed to send
                      else			// Server allowed to send HTML
                        {
                          hstat = 1;		// and set appropriate flag

#ifdef DBPRINT
                          if (_SCC_milter_config.debug_level > 20)
                            syslog(LOG_ERR,
                                "%s %s Remote server %s allowed to send HTML",
               				      localid,dblev[0],priv->mlfi_connectfrom);
#endif
                        }
                    else
                      {
                        hstat = 1;    // Global permission flag to receive HTML was set

#ifdef DBPRINT
                        if (_SCC_milter_config.debug_level > 20)
                          syslog(LOG_ERR,
                    	    "%s %s Global permissions to receive HTML",localid,dblev[0]);
#endif
                      }

// Check all recipients. When only one recipient will hasn't permissions to receive
// HTML messages, a message qualified as spam. When host or sender already allowed
// to send HTML messages, no additional checks issued.

                for(i = 1, scc1 = priv->mlfi_rcptto;
          		(i <= priv->recipients) && (!hstat);i++,scc1++)
                 if (!scc1->_uflags._htmlin)
                   {

#ifdef DBPRINT
                     if (_SCC_milter_config.debug_level > 20)
                       syslog(LOG_ERR,
                             "%s %s Recipient %s declined to receive HTML, uflags=0x%X",
                  	      localid,dblev[0],scc1->sccuser,*((int *) &scc1->_uflags));

                     free(dspsave);
#endif

                     return NORECVHTML;
                   }

#ifdef DBPRINT
                  else
                    if (_SCC_milter_config.debug_level > 20)
                      syslog(LOG_ERR,
                             "%s %s Recipient %s allowed to receive HTML, uflags=0x%X",
                  	       localid,dblev[0],scc1->sccuser,*((int *) &scc1->_uflags));
#endif

              }			// HTML isolation active
          }			// one part HTML message
         else      		// This is text message (non-html)
           {
#ifdef DBPRINT
             if (_SCC_milter_config.debug_level > 20)
               syslog(LOG_ERR,"%s %s Text media type: %s",localid,dblev[0],dspsave);
#endif
           }
       else			// Non-text (binary, etc.) part
         {
#ifdef DBPRINT
           if (_SCC_milter_config.debug_level > 20)
             syslog(LOG_ERR,"%s %s Non-text media type, Content-Type is: %s",
       							localid,dblev[0],dspsave);

           if (_SCC_milter_config.debug_level > 50)
             syslog(LOG_ERR,
      	 	"%s %s This is a non-text message or message part,cflags=0x%X,bflags=0x%X",
      	 	 	localid,dblev[1],*((u_short *) &priv->_cflags),
      	 	 				*((u_short *) &priv->_bflags));
#endif

// When enterprise mode filtering are active (--enterprise), that we should check
// available enterprise flags. Now is it one: allowing attachments. We will check
// enterprise violations with next conditions: enterprise & BODY & multipart OR
// enterprise & HEADER & !multipart (exclude occassion, when user can send one
// part message with non-text file)

           if (((_SCC_milter_config._idflags._enterprise) &&
           	(priv->_cflags._multipart) && (where == BODY)) ||
                  ((_SCC_milter_config._idflags._enterprise) &&
           	     (!priv->_cflags._multipart) && (where == HEADER)))

             {

// Checking permissions on attaches receiving from server

               if (priv->_cflags._attachfrom)    	       // Server allowed to receive
                 {
                   estat = 1;
                   sstat = 1;

#ifdef DBPRINT
                   if (_SCC_milter_config.debug_level > 20)
                     syslog(LOG_ERR,"%s %s Server %s allowed to send attachments",
                   			   localid,dblev[0],inet_ntoa(priv->inaddr));
#endif
                 }
                else

// Cehcking permissions on attaches receiving from user

                  if (priv->mlfi_mailfrom->_uflags._attach)   // User allowed to send
                    estat = 1;           // and set appropriate flag
                   else
                     estat = 0;		 // decline attachments

#ifdef DBPRINT
// Print info about allowed user, only when really allowing was about user, not
// about server

               if ((estat) && (!sstat))
                 if (priv->_cflags._localout)
                   if (_SCC_milter_config.debug_level > 20)
                     syslog(LOG_ERR,"%s %s Local user %s allowed to send attachments",
                   			localid,dblev[0],priv->mlfi_mailfrom->sccuser);
                    else ;
                  else
                    if (_SCC_milter_config.debug_level > 20)
                      syslog(LOG_ERR,"%s %s Remote user %s allowed to send attachments",
                   			localid,dblev[0],priv->mlfi_mailfrom->sccuser);
                     else ;
                else ;
#endif

// Check all recipients. When only one recipient will hasn't permissions to receive
// HTML messages, a message qualified as spam. When host or sender already allowed
// to send attachments, no additional checks issued. To succesful receive attachments
// sender and ALL recipients should be marked as 'attach'.
// BEWARE! When sender marked as 'locally out', receivers DOES NOT CHECK!

               for(i = 1, scc1 = priv->mlfi_rcptto;
          	     (i <= priv->recipients) && (!priv->_cflags._localout);i++,scc1++)
                if ((!scc1->_uflags._attach) || (!estat))
                  {

#ifdef DBPRINT
                    if (_SCC_milter_config.debug_level > 20)
                      if (!scc1->_uflags._attach)
                        syslog(LOG_ERR,
                             "%s %s Recipient %s declined to receive attachments, uflags=0x%X",
                  	      localid,dblev[0],scc1->sccuser,*((int *) &scc1->_uflags));
                       else
                         syslog(LOG_ERR,
                             "%s %s Recipient %s allowed, but sender/server disallowed to send attachments, uflags=0x%X",
                  	      localid,dblev[0],scc1->sccuser,*((int *) &scc1->_uflags));
                     else ;

                    free(dspsave);
#endif

                    return ENOATTACH;
                  }

#ifdef DBPRINT
                 else
                   if (_SCC_milter_config.debug_level > 20)
                     syslog(LOG_ERR,
                             "%s %s Recipient %s allowed to receive attachments, uflags=0x%X",
                  	       localid,dblev[0],scc1->sccuser,*((int *) &scc1->_uflags));
                    else ;
#endif

              }			// Enterprise active
         }			// Non-text (binary, etc.) part
    }				// If media type was set...

// When we have some parameters after Content-Type specification, we search "charset="
// We doing search only for text or multipart media

  if (((priv->_bflags._text) ||
  		(priv->_bflags._multipart) || (priv->_cflags._multipart))
  						&& (!priv->_bflags._noctparam))
    {
#ifdef DBPRINT
      if (_SCC_milter_config.debug_level > 20)
         syslog(LOG_ERR,
            "%s %s Text/multipart media type with params, searching \"charset=\"",
       								    localid,dblev[0]);
#endif

      if ((pt = (strncasestr(ptsave + 1,"charset",speclen - (ptsave - line) - 1))) != 0)
        {
          if ((_SCC_milter_parse_content_type(ctx,CHARSET,pt)) == ERR)
            {
#ifdef DBPRINT
              free(dspsave);
#endif
              return INVMSCTYPE;
            }

// We need only one "charset" repository, because we checking charsets consequently,
// when invalid charset will be found, letter will stopped.

          priv->_cflags._charset = OK;

// When we received a message, where 'charset=""' is (I know, this is a bug, but I have
// seen these messages)

          if (!strlen(priv->charset))
            {

#ifdef DBPRINT
              if (_SCC_milter_config.debug_level > 20)
	        syslog(LOG_ERR,
	    	 "%s %s We found zero-length charset specification, assuming no charset",
	    								localid,dblev[0]);
#endif
              priv->_cflags._charset = NO;
            }
           else
             if (_SCC_milter_check_charset(ctx,priv->charset))
               {
#ifdef DBPRINT
                 free(dspsave);
#endif
                 return sccerror;	// Implicit was set by check_charset()
               }
        }   				// Charset specification was found
    }					// Text media type and parameters presented

// We have received multipart message (Content-Type: multipart/...). Here we search
// and store boundary indentation. When boundary was found we format real boundary
// keep in mind, REAL boundary ALWAYS begin from "--", even nothing follows!
// We can called from header or from body, so we should check, who called us

  if (((where == HEADER) && (priv->_cflags._multipart)) ||
  			((where == BODY) && (priv->_bflags._multipart)))
    {

// Checking on succesfull parsing multipart definition. When failed, return error.
// Checking only when parameters are exist (; after media type was found)

      if (((where == HEADER) && (!priv->_cflags._noctparam)) ||
  			((where == BODY) && (!priv->_bflags._noctparam)))
        {
#ifdef DBPRINT
          if (_SCC_milter_config.debug_level > 20)
            syslog(LOG_ERR,
            	"%s %s Multipart with params, searching \"boundary=...\"",
            							localid,dblev[0]);
#endif

          if ((pt = (strncasestr(ptsave + 1,
      			"boundary",speclen - (ptsave - line) - 1))) != 0)
            {

// TODO: Add more smart processing nested multipart messages

//              if (((where == HEADER) && (!priv->_cflags._boundary)) ||
//  				((where == BODY) && (!priv->_bflags._boundary)))
              if (!priv->_cflags._boundary)
                {
#ifdef DBPRINT
                  if (_SCC_milter_config.debug_level > 20)
	            syslog(LOG_ERR,
	              "%s %s This will a first boundary setting",localid,dblev[0]);
#endif

                  if ((_SCC_milter_parse_content_type(ctx,BOUNDARY,pt)) == ERR)
                    {
#ifdef DBPRINT
                      free(dspsave);
#endif
                      return INVMSCTYPE;
                    }
                   else
                     {
                       memmove(priv->boundary + 2,
                       			priv->boundary, strlen(priv->boundary));
                       memmove(priv->boundary,"--",2);

// Keep in mind - boundary can be as in whole messahe (in header), as in message part
// (body). This goal makes engine slightly monstrous, but reilable.

                       if (where == HEADER)
                         priv->_cflags._boundary = OK;
                        else
                          priv->_bflags._boundary = OK;
                     }
                }			// if (!boundary)...
               else
                 {
#ifdef DBPRINT
                   if (_SCC_milter_config.debug_level > 20)
	             syslog(LOG_ERR,
	                "%s %s This will a non-first boundary setting, see TODO",
	            						  localid,dblev[0]);
#endif
                 } 		// if first boundary set...
            }			// if "boundary=..." found
        }			// if !noctparam ...
    }				// if multipart...

// When debug is active, do not forget free allocated memory for debugging purposes

#ifdef DBPRINT
  free(dspsave);
#endif

  return NOERROR;
}

/* ---------------------------------------------------------------------------------
 *
 *	Parse Content-Type line snd select some part from it.
 *
 *	Call with:	ctx  	- communications area for sendmail
 *			header 	   - pointer to searched parameter description
 *			source	   - source line (NO '\0' cokring!)
 *
 *	Return code:   >0    -  line parsed succesfully, this is a field length
 *		       -1    -  parsing errors were occured
 *
 * --------------------------------------------------------------------------------- */

int _SCC_milter_parse_content_type(SMFICTX *ctx,char header,char *source)
{
  struct mlfiPriv *priv = MLFIPRIV;
  char *pta,*ptn,**item;
  u_char i,j,ok = 0, ischarset = 0;
  int item2length;
  u_short *ilength,plen[NSTOPS];

#ifdef DBPRINT
  char *localid="(parsecontent)";
  char *itemdesc,*dspsave;
#endif

// First, we calculate, what header we should parse

  if (header == CHARSET)
    {
      item = &priv->charset;
      ilength = &priv->charsetlength;
      ischarset = 1;
#ifdef DBPRINT
      itemdesc = "charset";
#endif
    }
   else
     {
       item = &priv->boundary;
       ilength = &priv->boundarylength;
#ifdef DBPRINT
       itemdesc = "boundary";
#endif
     }

// We search a ';' char (RFC 2045 defines it as parameters separator).
// Also we search '\n' and '\0' chars and storing pointers to all found stoppers
// Char claimed as "separator" only when it follows with '"' char for ';' and '\0'
// separators and when it follows with '"' and '\r' for '\n' separator

  for(i = 0;i < NSTOPS; i++, ok = 0)
   {
     ptn = memchr(source, nstop[i], WORKLENGTH);

// We search current separator, when not found - we hasn't find it and go to search
// next separator

     if (ptn == NULL)
       {
         ok = 1;

#ifdef DBPRINT
         if (_SCC_milter_config.debug_level > 100)
           syslog(LOG_ERR,
           	"%s %s %s separator was not found", localid,dblev[2],cnstop[i]);
#endif
         continue;
       }

// We check separator - when it ';' or '\0' and follows with any, NOT '"' -
// it is not a separator, when it is a '\n' char, it should follow by '"' and '\r'
// When this is a charset spec, '"' can missed, with boundary CANNOT

     if (nstop[i] != '\n')
       if ((!ischarset) && ((*(ptn - 1)) != '"'))
         {
           ok = 1;

#ifdef DBPRINT
           if (_SCC_milter_config.debug_level > 100)
             syslog(LOG_ERR,"%s %s nstop=0x%X,ptn-1=0x%X",
         		localid, dblev[2], (int) nstop[i], (int) (*(ptn-1)));
#endif

         }

      if (nstop[i] == '\n')
	if ((*(ptn - 1)) == '\r')
          if (((*(ptn - 2)) != '"') && (!ischarset))
            {
              ok = 1;

#ifdef DBPRINT
              if (_SCC_milter_config.debug_level > 100)
                syslog(LOG_ERR,"%s %s nstop=0x%X,ptn-1=0x%X,ptn-2=0x%X", localid,
       		      dblev[2],(int) nstop[i], (int) (*(ptn - 1)),(int) (*(ptn - 2)));
#endif

       	    }
           else ;
         else
          if ((*(ptn - 1)) != '"')
            {
              ok = 1;

#ifdef DBPRINT
              if (_SCC_milter_config.debug_level > 100)
                syslog(LOG_ERR,"%s %s nstop=0x%X,ptn-1=0x%X,ptn-2=0x%X", localid,
       		      dblev[2],	(int) nstop[i], (int) (*(ptn - 1)),(int) (*(ptn - 2)));
#endif

       	    }
           else ;
       else ;

// When we claimed this char as separator, we mark this

     if (!ok)
       {
         plen[i] = ptn - source;

#ifdef DBPRINT
         if (_SCC_milter_config.debug_level > 100)
           syslog(LOG_ERR,"%s %s %s separator was found at 0x%X,length is %d",
         				localid,dblev[2],cnstop[i],(int) ptn,plen[i]);
#endif

       }
      else
        {
          plen[i] = SHRT_MAX;

#ifdef DBPRINT
         if (_SCC_milter_config.debug_level > 100)
           syslog(LOG_ERR,"%s %s %s separator was claimed as ordinary char",
           						localid,dblev[2],cnstop[i]);
#endif

        }		// if (!ok)
   }			// for(i < NSTOPS)

// We look about all stoppers and count, which weren't found

  for(i = j = 0;i < NSTOPS; i++)
   if (plen[i] == SHRT_MAX)
     j++;

// When any stoppers wasn't found, this is a BUUUUUG!

  if (j == 3)
    {

#ifdef DBPRINT
      if (_SCC_milter_config.debug_level > 100)
        syslog(LOG_ERR,"%s %s None of stoppers were found, parsing abort!",
        							localid,dblev[2]);
#endif

      return ERR;
    }

// Else we taking smallest and counting itemlength

  for(i = 0,item2length = SHRT_MAX;i < NSTOPS;i++)
   {

#ifdef DBPRINT
     if (_SCC_milter_config.debug_level > 100)
       syslog(LOG_ERR,"%s %s i=%d,itemlength=%d,plen[i]=%d",
     				localid,dblev[2],i,item2length,plen[i]);
#endif

     if (item2length > plen[i])
       {

#ifdef DBPRINT
         if (_SCC_milter_config.debug_level > 100)
           syslog(LOG_ERR,"%s %s Reassign %d to %d on %d round",
         			localid, dblev[2],item2length,plen[i],i);
#endif

         item2length = plen[i];
       }
   }

#ifdef DBPRINT

// We selecting this smallest value

  if (_SCC_milter_config.debug_level > 20)
    syslog(LOG_ERR,"%s %s We selecting smallest length, it is %d bytes",
    						localid,dblev[0],item2length);
#endif

// When memory for save item hasn't taken yet, do this. Else simply clean this area
// Also, when we taking memory, we keep taken size (and use it in clean process)
// Also, when allocated repository is bigger, than new, reallocate memory

  if (*item == NULL)
    {
      *item = (char *) calloc(item2length + 1,1);
      *ilength = item2length + 1;

#ifdef DBPRINT
      if (_SCC_milter_config.debug_level > 100)
        syslog(LOG_ERR,"%s %s Allocating %d bytes for %s",
        			localid,dblev[2],item2length,itemdesc);
#endif

    }
   else
     {
       if ((*ilength - item2length) <= 1)
         {
           *item = (char *) realloc(*item,item2length + 1);
           *ilength = item2length + 1;

#ifdef DBPRINT
           if (_SCC_milter_config.debug_level > 100)
             syslog(LOG_ERR,"%s %s Reallocating %d bytes for %s",
        				localid,dblev[2],item2length,itemdesc);
#endif
         }

       bzero(*item,*ilength);
     }

#ifdef DBPRINT
  dspsave = (char *) calloc(item2length + 10,1);
  memmove(dspsave,source,item2length);

  if (_SCC_milter_config.debug_level > 20)
    syslog(LOG_ERR,"%s %s %s specification was found",localid,dblev[0],itemdesc);

  if (_SCC_milter_config.debug_level > 100)
    syslog(LOG_ERR,"%s %s source->%s,header=%s,item=0x%X,itemlength=%d",
    				localid,dblev[2],dspsave,itemdesc,(int) *item,item2length);
#endif

// We search now a '=' char in line. RFC 2045 described it as delimitter in parameters
// But, when we cannot find it, we try to search ';' char. It violated RFC 2045, but
// some strange mailers can use it.

  if ((pta = memchr(source,'=',item2length)) == NULL)
    if ((pta = memchr(source,':',item2length)) == NULL)
      {

#ifdef DBPRINT
        if (_SCC_milter_config.debug_level > 50)
          syslog(LOG_ERR,"%s %s Invalid %s specification was detected: %s",
        					  localid,dblev[1],itemdesc,source);
#endif

        return ERR;
      }
     else
       {

#ifdef DBPRINT

         if (_SCC_milter_config.debug_level > 20)
           syslog(LOG_ERR,"%s %s ':' found instead of '=' in %s, line is: %s",
       						localid,dblev[0],itemdesc,source);
#endif
         pta++;
       }
   else
     pta++;

// We now copy charset specification in work area, and drop quotes, when detect
// Also we unescape this line against spaces (when mail contained violated RFC 2045
// lines, probably after ';' can followed space

  memcpy(*item, pta, item2length - (pta - source));

// We shoul unescape line from spaces ONLY when is it charset=..., not boundary=...
// (many modern agents want spaces inside a boundary)

  if (ischarset)
    strunescape(*item,' ');

// Unesacpe line against '\r' (F*CKED Dos/Win heritage!)

  strunescape(*item,'\r');

// Recalculate item length (we now does not contain "item=" text, only item value

  *ilength = strlen(*item);

// Unquote charset, when occured (in RFC 2045 is - quoted and unquoted charsets are
// equal, so we should follows this way. Also this time we should recalculate length
// of item line (it is contains now only item specification, not "item=" text
// itself)

  if (**item == '"')
    {
      memmove(*item, *item + 1, strlen(*item) - 1);
      pta = strchr(*item,'"');
      *pta = '\0';
      *ilength = strlen(*item);
    }

#ifdef DBPRINT
  if (_SCC_milter_config.debug_level > 20)
    syslog(LOG_ERR,
  	"%s %s We store %s as %s, pta=0x%X, real length=%d",
  			localid,dblev[0],*item,itemdesc,(int) pta,*ilength);

  free(dspsave);
#endif

  return NOERROR;
}
