/*-
 * Copyright (c) 2002  Granch Ltd. 	All rights reserved.
 *
 * All or some portions of this file are derived from material licensed
 * to the Grahcn Ltd. and are reproduced herein with the permission of
 * Granch Ltd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE GRANCH LTD. AND THEIR CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author : Rashid N. Achilov		E-Mail shelton@granch.ru
 *
 *	$Id: sccmilter.c,v 0.94.2.3 2005/10/06 10:24:22 shelton Exp $
 */
/*------------------------------------------------------------------------

        Spam Control and Canceling mail filter, based on libmilter API

 	UNIX FreeBSD 5.4 version

        Main program.

 	This program based on:
 	LibMilter API sample program (sample.c)

  	This program REQUIRED a GNU getopt package. You can install it from
  	/usr/ports/devel/libgnugetopt. Also, of cource, this program required
        sendmail :-)

--------------------------------------------------------------------------*/
#include "sccmilter.h"
#include "externals.h"
#include "functions.h"

// Main program section

int main(int argc, char *argv[])
{
  char *p;
  int retcode;
  struct stat st,sb;			// Stat file struct
  FILE *pidf;				// PID file to save

#ifdef DBPRINT
  char *localid="(init)";
#endif

// config_ent structures array, which describes parameters to parsing config
// file at whole.
// Contains fields:
// Name - parameter name (up to 32 chars)
// Exit - processor's code
// Valid - sing of validity parameter,
// HexMask - to properly setup a bit-field parameter (contains a mask, which will
// 	     OR'ed with current signs byte, do not use to other types)
// Options - Pointer to pointer to options list, used with this paramter
// Default - Default value for this parameter (if any)
// Which - address, which subroutine should place parameter value (void pointer!).
// All string variables should be (in structure) BEFORE all non-string (decimal, bool)
// variables.

  struct config_ent paramConfig[] =
     {

       { "ProgramName", STDCHAR, NULL, NULL, NULL, "sccmilter", &_SCC_milter_config.progname },
       { "ProgramVersion", STDCHAR, NULL, NULL, NULL, PKGVERSION, &_SCC_milter_config.progversion },
       { "SendmailPipe", STDCHAR, NULL, NULL, NULL, "/var/run/sccmilter", &_SCC_milter_config.sendmail_pipe },
       { "SCCHostsFile", STDCHAR, NULL, NULL, NULL, NULL, &_SCC_milter_config.scchosts },
       { "SCCUsersFile", STDCHAR, NULL, NULL, NULL, NULL, &_SCC_milter_config.sccusers },
       { "PIDFile", STDCHAR, NULL, NULL, NULL, "/var/run/sccmilter.pid", &_SCC_milter_config.pidfile },
       { "IsolationPath", STDCHAR, NULL, NULL, NULL, "/tmp", &_SCC_milter_config.isolation },
       { "ExtraSavePath", STDCHAR, NULL, NULL, NULL, "/tmp", &_SCC_milter_config.extrasave },
       { "AllowedCodepages", STDCHAR, NULL, NULL, NULL, NULL, &_SCC_milter_config.codepages },
       { "LocalDomain", STDCHAR, NULL, NULL, NULL, NULL, &_SCC_milter_config.domain },
       { "NoCharsetIsSpam", STDBOOL, NULL, 0x80, NULL, NULL, &_SCC_milter_config._idflags },
       { "SpamAction", STDLIST, NULL, NULL, &act_modes, NULL, &_SCC_milter_config.mode },
       { "EnterpriseAction", STDLIST, NULL, NULL, &act_modes, NULL, &_SCC_milter_config.emode },
       { "SendmailTimeout", STDINT, NULL, NULL, NULL, "300", &_SCC_milter_config.sendmail_timeout },
       { "DebugLevel", STDINT, NULL, NULL, NULL, "0", &_SCC_milter_config.debug_level },
       { "DaemonMode", STDBOOL, NULL, 0x1, NULL, NULL, &_SCC_milter_config._idflags },
       { "EnterpriseActivate", STDBOOL, NULL, 0x40, NULL, NULL, &_SCC_milter_config._idflags },
       { "IsolateHTML", STDLIST, NULL, NULL , &html_modes, NULL, &_SCC_milter_config.hmode },
       { "Null", NULL, NULL, NULL, NULL, NULL, NULL },

     };

// Catch-up signal handling

  signal(SIGINT,_SCC_milter_close);
  signal(SIGTERM,_SCC_milter_close);
  signal(SIGHUP,SIG_IGN);

// Store pointer to program name (when we invoked with full path, drop pathname)

  p = strrchr(argv[0],'/' );

  if (p == NULL)
    progname = argv[0];
   else
     progname = p + 1;

// Parsing configuration file. We can setup configuration file location only in
// compilation time

  parseconfig(CONFIG, paramConfig);

// Process short or long options

  _SCC_milter_parse_commandline(argc,argv);

// Check command-line specified parameters and values. When checknig
// sendmail timeout, call appropriate procedure to set it. When timeout in command-line
// greather than zero, store it in parameters structure, else use config value

  if (_SCC_milter_config.sendmail_timeout < 0)
    {
     fputs("Sendmail connect timeout must be greather than zero",stderr);
      _SCC_milter_close(EX_USAGE);
    }

  if (smfi_settimeout(_SCC_milter_config.sendmail_timeout) == MI_FAILURE)
    {
      fputs("Set timeout to sendmail connect (smfi_settimeout) failed",stderr);
      _SCC_milter_close(EX_USAGE);
    }

// Check specified temporary directory against exist, directory and
// permissions.

  if (stat(_SCC_milter_config.isolation,&st) != ERR)
    if (!S_ISDIR(st.st_mode))
      {
        fprintf(stderr,"%s is not a directory, removed\n",_SCC_milter_config.isolation);
        unlink(_SCC_milter_config.isolation);
      }

  if ((access(_SCC_milter_config.isolation, 0700) == ERR) &&
  			(strcmp(_SCC_milter_config.isolation,"/tmp")))
    {
      fprintf(stderr,"Wrong permissions on the %s : %s, removed\n",
      				_SCC_milter_config.isolation,strerror(errno));
      unlink(_SCC_milter_config.isolation);
    }

// Check specified extra save directory against exist, directory and
// permissions.

  if (stat(_SCC_milter_config.extrasave,&st) != ERR)
    if (!S_ISDIR(st.st_mode))
      {
        fprintf(stderr,"%s is not a directory, removed\n",_SCC_milter_config.extrasave);
        unlink(_SCC_milter_config.extrasave);
      }

  if ((access(_SCC_milter_config.isolation, 0700) == ERR) &&
  			(strcmp(_SCC_milter_config.isolation,"/tmp")))
    {
      fprintf(stderr,"Wrong permissions on the %s : %s, removed\n",
      				_SCC_milter_config.extrasave,strerror(errno));
      unlink(_SCC_milter_config.extrasave);
    }

// Parse SCC hosts and SCC users file. When parsing one of these files fail, stop
// immediately.

 if ((scchosts =
 	_SCC_milter_parse_hosts_file(_SCC_milter_config.scchosts,&qscchosts)) == 0)
   _SCC_milter_close(EX_TEMPFAIL);

 if ((sccusers =
 	_SCC_milter_parse_users_file(_SCC_milter_config.sccusers,&qsccusers)) == 0)
   _SCC_milter_close(EX_TEMPFAIL);

// When requested, go into daemon mode

  if (_SCC_milter_config._idflags._isdaemon)
    _SCC_milter_daemon_mode();

// Open system log and send start identification line

  if (!_SCC_milter_config._idflags._isdaemon)
    openlog(progname, LOG_PID | LOG_PERROR, LOG_MAIL);
   else
     openlog(progname, LOG_PID, LOG_MAIL);

  _flags._openlog = OK;

// Now we really starting filter ;-)

  syslog(LOG_INFO,"starting sendmail filter '%s' version %s, patch 5",
  	   _SCC_milter_config.progname,_SCC_milter_config.progversion);

// Store PID number of daemonized process

  if (stat(_SCC_milter_config.pidfile,&sb) > NULL)
    {
      unlink(_SCC_milter_config.pidfile);
      syslog(LOG_INFO,"Old PID file was detected - unclean shutdown?");
    }

  if ((pidf = (fopen(_SCC_milter_config.pidfile,"w"))) == NULL)
    {
      syslog(LOG_ERR,"PID file %s cannot open : %m",_SCC_milter_config.pidfile);
      _SCC_milter_close(EX_TEMPFAIL);
    }
   else
     {
       _flags._pidfile = OK;

#ifdef DBPRINT
#warning This version is for developing only, LOTS of debug info will be generated
       if (_SCC_milter_config.debug_level > 20)
         syslog(LOG_ERR,"%s %s PID file open, name %s, flag %d",localid,dblev[0],
         			        _SCC_milter_config.pidfile,_flags._pidfile);
#endif

       fprintf(pidf,"%d",(int) getpid());
       fclose(pidf);
     }

// Set debug level and print startup parameters

  smfi_setdbg(_SCC_milter_config.debug_level);

  p = calloc(10,1);
  strtoupper(p,act_modes[*((char *) &_SCC_milter_config.mode)]);
  syslog(LOG_INFO,"action with spam suspected - %s",p);

// Also print enterprise mode, when enterprise checks allowed

  if (_SCC_milter_config._idflags._enterprise)
    {
      syslog(LOG_INFO,"enterprise violations check allowed");

      strtoupper(p,act_modes[*((char *) &_SCC_milter_config.emode)]);
      syslog(LOG_INFO,"action with etnerprise violation mail - %s",p);
    }

// Also notify, when HTML isolation mode is active

  if (_SCC_milter_config.hmode._incoming)
    syslog(LOG_INFO,"Incoming HTML messages isolation mode active");

  if (_SCC_milter_config.hmode._outgoing)
    syslog(LOG_INFO,"Outgoing HTML messages isolation mode active");

// When we requested keep all mail, warn about

  if (_flags._nodelete)
    syslog(LOG_INFO,"Any mail will be NOT deleted from spam isolator");

// When we requested keep spam mail, warn about

  if (_flags._spam_nodelete)
    syslog(LOG_INFO,"SPAM mail will be NOT deleted from spam isolator");

// When we requested imitation mode, warn about

  if (_flags._imitate)
    syslog(LOG_INFO,"IMITATION MODE! Nothing actions will be performed!");

// Print parameters, when debug requested

#ifdef DBPRINT
  if (_SCC_milter_config.debug_level > 20)
    syslog(LOG_INFO,"%s %s debug level: %d, sendmail timeout: %d",localid,dblev[0],
      	         _SCC_milter_config.debug_level, _SCC_milter_config.sendmail_timeout);
#endif

// Set reasonable umask

  umask(0027);

// Check and remove, when need old local communication socket file

  if (stat(_SCC_milter_config.sendmail_pipe,&st) != ERR)
    unlink(_SCC_milter_config.sendmail_pipe);

// Set connection port

  if (smfi_setconn(_SCC_milter_config.sendmail_pipe) == MI_FAILURE)
    {
      syslog(LOG_ERR,"Set connection port (smfi_setconn) failed,"
      					"port %s",_SCC_milter_config.sendmail_pipe);
      _SCC_milter_close(EX_SOFTWARE);
    }

// Register this filter in sendmail space

  if (smfi_register(sccfilter) == MI_FAILURE)
    {
      syslog(LOG_ERR,"sccfilter registration (smfi_register) failed");
      _SCC_milter_close(EX_UNAVAILABLE);
    }

// Main SMFI_ call

  if ((retcode = smfi_main()) != MI_SUCCESS)
    syslog(LOG_ERR,"Main SMFI_ call (smfi_main) failed, return code %d",retcode);

  return retcode;
}
