/*-
 * Copyright (c) 2002  Granch Ltd. 	All rights reserved.
 *
 * All or some portions of this file are derived from material licensed
 * to the Grahcn Ltd. and are reproduced herein with the permission of
 * Granch Ltd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE GRANCH LTD. AND THEIR CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author : Rashid N. Achilov		E-Mail shelton@granch.ru
 *
 *	$Id: utilites.c,v 0.94 2005/10/04 12:11:54 shelton Exp $
 */
/*------------------------------------------------------------------------

        Spam Control and Canceling mail filter, based on libmilter API

 	UNIX FreeBSD 5.4 version

        Various utilites.

--------------------------------------------------------------------------*/
#include "sccmilter.h"
#include "externals.h"
#include "functions.h"

/* ---------------------------------------------------------------------------------
 *
 *	Usage program printing (call when invalid commandline specified)
 *
 *	Call with:	none
 *
 *	Return code:	none
 *
 * --------------------------------------------------------------------------------- */

void _SCC_milter_usage (void)
{
  fputs("Usage: [-c socket] [-d] [-D debug_level] [-n] [-hv] [-o domain]\n",stderr);
  fputs("\t [-p tmpdir] [-t timeout] [-m {isolate|discard}] [-g codepage, codepage...]\n",stderr);
  fputs("\t [-a extrasave dir]\n",stderr);
  fputs("\t [-P pidfile] [--spam-no-delete] -e {reject|isolate|isoreject}]",stderr);
  fputs("\t [-l {no|full|incoming|outgoing}] [--imitate] [--no-delete]",stderr);

  fprintf(stderr,"\n\n\
  -h | --help                     - prints this page\n\
  -v | --version		  - prints the version information only\n\
  -c | --sendmail sendmail_socket - sendmail communication socket\n\
   				      (default /var/run/sccmilter)\n\
  -d | --daemon                   - daemon mode\n\
  -D | --debug value              - turn on debugging [0...110]\n\
  -p | --tmpdir directory         - tempopary directory path (default is /tmp)\n\
  -a | --extrasave directory      - directory path to extra saves mail (default is /tmp)\n\
  -t | --sendmail-timeout time    - sendmail connection timeout in seconds\n\
  -m | --mode\n\
       {reject|discard|isolate}   - reject/discard/isolate spam messages\n\
  -n | --enterprise		  - activate enterprise violation checks\n\
  -e | --emode\n\
       {reject|discard|isolate}   - reject/discard/isolate\n\
       				    messages with enterprise violations\n\
  -l | --nohtml\n\
      {no|full|incoming|outgoing} - isolate HTML messages: full/incoming/outgoing\n\
       --imitate		  - all actions will be imitated only\n\
       --no-delete		  - no delete any mail anyway\n\
       --spam-no-delete		  - no delete spam mail anyway\n\
  -g | --codepage page1,page2,... - allowed mail only in page1,page2,... codepages\n\
  -o | --domain domain		  - local domain for this box\n\
  -P | --pidfile file		  - full PID file name\n\
  				      (default /var/run/sccmilter.pid)\n\n");

  fprintf(stderr,"Package: %s-%s\n%s\n%s, version %s\n",
             			PKGNAME,PKGVERSION,SCC_MILTER,SCC_DATE,PKGVERSION);
  return;
}

/* ---------------------------------------------------------------------------------
 *
 *	Traverse to daemon mode (call when daemonized)
 *
 *	Call with:	none
 *
 *	Return code:	none
 *
 * --------------------------------------------------------------------------------- */

void _SCC_milter_daemon_mode(void)
{

// When parent isn't init, we can safely ignore background in/out signals

  if (getppid() != 1)
    {
      signal(SIGTTOU, SIG_IGN);
      signal(SIGTTIN, SIG_IGN);
      signal(SIGTSTP, SIG_IGN);
    }

// Go to daemon mode

  if (fork() != NULL)
    exit(0);

// This stream now is group and session leader, but without control terminal

  setsid();

// Close standard streams (stdin, stdout, stderr)

  close(0);
  close(1);
  close(2);

// Change work directory

  chdir("/");

  return;
}

/* ---------------------------------------------------------------------------------
 *
 *	Exit from filter (SIGINT was arrived)
 *
 *	Call with:	exitcode	- exit code for transfer to shell
 *
 *	Return code:	exitcode	- exit code to transfer to shell
 *
 * --------------------------------------------------------------------------------- */

void _SCC_milter_close(int exitcode)
{
  if (!exitcode)
    syslog(LOG_INFO,"stopping sendmail filter '%s' (process '%s')",
  					sccfilter.xxfi_name,progname);
   else
     syslog(LOG_INFO,"aborting sendmail filter '%s' (process '%s')",
  					sccfilter.xxfi_name,progname);

  if (_flags._pidfile)			// Delete PID file
    unlink(_SCC_milter_config.pidfile);

  if (_flags._openlog)			// Close system log
    closelog();

  exit(exitcode);
}

/* ---------------------------------------------------------------------------------
 *
 *	Parsing SCC hosts file (call when configuration file scchosts parsed)
 *
 *	Call with:	filename	- file name of SCC hosts file
 *			qrecs		- pointer to place quantity of parsed records
 *
 *	Return code:	<= 0		- error indicated
 *			> 0		- quantity of parsed records
 *
 * --------------------------------------------------------------------------------- */

struct scchost *_SCC_milter_parse_hosts_file(char *filename, u_short *qrecs)
{
  int work,bla,recs = 0;
  struct stat sb;
  char *buf,*bufx,*line,*posix,*pos,*poz,*poza,**pozb;
  struct scchost *scc1,*scc2;
  struct hostent *hp;

#ifdef DBPRINT
  char *localid = "(utility)";
#endif

// Open config file

  if ((work = open(filename,O_RDONLY)) > 0)
    {
      stat(filename,&sb);
      bufx = (char *) calloc(sb.st_size * 2, 1);
      buf = bufx + sb.st_size;

      read(work,bufx,sb.st_size);	// Fucked Feof()!
      close(work);

// Now we drop lines with comment char at first position and strip spaces.
// After this procedure in output buffer lies only name=value pairs, separa-
// ted by '\n' char

      if ((bla = readconfig('#',bufx,buf,sb.st_size)) == ERR)
	{
	  fprintf(stderr,"cannot read SCC hosts file %s\n",filename);
          free(bufx);
          return(0);
	}

// scchosts structures array is one more, than scchosts file contents, so we should
// calculate lines quantity. Each turn we shorting checked length to already checked
// part (posix is a finish of part, pos is a start, bla - common length)

      for(pos = posix = buf;(pos - buf) <= bla;pos = posix + 1)
       if ((posix = memchr(pos,'\n',bla - (posix - pos))) != NULL)
         recs++;
        else
          break;

// Store in external variable address of area to keeping parsed-down file

      scc1 = (struct scchost *) calloc(recs + 1,sizeof(struct scchost));

// Reading filtered buffer line-by-line. Now we search '\n' char and at each
// loop round shift position on 1 greater than searched position. Loop stops
// when length filtered buffer will be exhausted.

      for(scc2 = scc1, line = posix = buf;(line - buf) <= bla;line = posix + 1)
       {

// When we suddenly reached a finish of the array, stop working, else boundary line
// to checking in string functions and error message

	 if ((posix = memchr(line,'\n',bla - (posix - line))) == NULL)
           break;
          else
            *posix = '\0';

// Search ':' char and taking string BEFORE it as host name

	 if ((pos = memchr(line,':',(posix - line))) != NULL)
           *pos = '\0';
          else
            {
              fprintf(stderr,"cannot find separator in host line %s, aborted\n",line);
              free(bufx);
              return(0);
            }

// Resolve this address. When resolving failed, this address will be skip

         if ((hp = gethostbyname(line)) == 0)
           {
             fprintf(stderr,"cannot resolve host %s, skipped",line);
             continue;
           }

         memmove(&scc2->scc_addr.s_addr,hp->h_addr,hp->h_length);

// Now we parsing a parameters line: using strsep we parse sigls line and setup
// correspond bit sign, when it found in line

         poz = pos + 1;

         for(pozb = &poz;(poza = strsep(pozb,",")) != 0;)
          {

// Set a "html allowed" flag

            if ((strcmp(poza,"html")) == 0)
              {
                scc2->_hflags._html = OK;
                continue;
              }

// Set a "attaches allowed" flag

            if ((strcmp(poza,"attach")) == 0)
              {
                scc2->_hflags._attach = OK;
                continue;
              }
          }

         scc2++;	// we cannot increment for empty structures :-)
       }		// Parsing host file main loop

#ifdef DBPRINT
      if (_SCC_milter_config.debug_level > 20)
        fprintf(stderr,
           "%s %s SCC hosts file has been parsed, parsed %d records\n",
           						localid,dblev[0],recs);
#endif

      free(bufx);
    }			// if ... open

// Specified configuration file didn't found

   else
     {
       fprintf(stderr,"opening file %s error: %s\n",filename,strerror(errno));
       return(0);
     }

// Store hosts quantity

  *qrecs = recs;

// When we here, all OK

  return scc1;
}

/* ---------------------------------------------------------------------------------
 *
 *	Parsing SCC hosts file (call when configuration file scchosts parsed)
 *
 *	Call with:	filename	- file name of SCC hosts file
 *			qusers		- pointer to place quantity of parsed records
 *
 *	Return code:	0		- error indicated
 *			!= 0		- address of SCC users parsed array
 *
 * --------------------------------------------------------------------------------- */

struct sccuser *_SCC_milter_parse_users_file(char *filename, u_short *qusers)
{
  int work,bla,recs = 0;
  struct stat sb;
  char *buf,*bufx,*line,*posix,*pos,*poz,*poza,**pozb;
  struct sccuser *scc1,*scc2;

#ifdef DBPRINT
  char *localid = "(utility)";
#endif

// Open config file

  if ((work = open(filename,O_RDONLY)) > 0)
    {
      stat(filename,&sb);
      bufx = (char *) malloc(sb.st_size * 2);
      buf = bufx + sb.st_size;

      read(work,bufx,sb.st_size);	// Fucked Feof()!
      close(work);

// Now we drop lines with comment char at first position and strip spaces.
// After this procedure in output buffer lies only name=value pairs, separa-
// ted by '\n' char

      if ((bla = readconfig('#',bufx,buf,sb.st_size)) == ERR)
	{
	  fprintf(stderr,"cannot read SCC users file %s\n",filename);
          free(bufx);
          return(NULL);
	}

// sccusers structures array is one more, than scchosts file contents, so we should
// calculate lines quantity. Each turn we shorting checked length to already checked
// part (posix is a finish of part, pos is a start, bla - common length)

      for(pos = posix = buf;(pos - buf) <= bla;pos = posix + 1)
       if ((posix = memchr(pos,'\n',bla - (posix - pos))) != NULL)
         recs++;
        else
          break;

// Store in external variable address of area to keeping parsed-down file

      scc1 = (struct sccuser *) calloc(recs + 1,sizeof(struct sccuser));

// Reading filtered buffer line-by-line. Now we search '\n' char and at each
// loop round shift position on 1 greater than searched position. Loop stops
// when length filtered buffer will be exhausted.

      for(scc2 = scc1, line = posix = buf;(line - buf) < bla;line = posix + 1)
       {

// When we suddenly reached a finish of the array, stop working, else boundary line
// to checking in string functions and error message. When we have got a empty line
// (we checking it in parseconfig(), but not all can eliminate) - skip it

	 if ((posix = memchr(line,'\n',bla - (posix - line))) == NULL)
           break;
          else
            *posix = '\0';

// Search ':' char and taking string BEFORE it as user name. Than, we take username
// AND zero-pad and place it in structure

	 if ((pos = memchr(line,':',(posix - line))) != NULL)
           *pos = '\0';
          else
            {
              fprintf(stderr,"cannot find separator in host line %s, aborted\n",line);
              free(bufx);
              return(0);
            }

         strcpy(scc2->sccuser,line);

// When username dose not include '@' char, we add here local domain name

         if ((strchr(scc2->sccuser,'@')) == 0)
           {
             strcat(scc2->sccuser,"@");
             strcat(scc2->sccuser,_SCC_milter_config.domain);
           }

// And finally, add angle brackets

         memmove(scc2->sccuser + 1,scc2->sccuser,strlen(scc2->sccuser));
         *(scc2->sccuser) = '<';
         *(scc2->sccuser + strlen(scc2->sccuser)) = '>';

// Now we parsing a parameters line: using strsep we parse sigls line and setup
// correspond bit sign, when it found in line

         poz = pos + 1;

         for(pozb = &poz;(poza = strsep(pozb,",")) != 0;)
          {

// Set a "html incoming allowed" flag

            if ((strcmp(poza,"htmlin")) == 0)
              {
                scc2->_uflags._htmlin = OK;
                continue;
              }

// Set a "html outgoing allowed" flag

            if ((strcmp(poza,"htmlout")) == 0)
              {
                scc2->_uflags._htmlout = OK;
                continue;
              }

// Set a "pgp allowed" flag

            if ((strcmp(poza,"pgp")) == 0)
              {
                scc2->_uflags._pgp = OK;
                continue;
              }

// Set a "executable attacments allowed" flag

            if ((strcmp(poza,"exattach")) == 0)
              {
                scc2->_uflags._exattach = OK;
                continue;
              }

// Set a "attachments allowed" flag

            if ((strcmp(poza,"attach")) == 0)
              {
                scc2->_uflags._attach = OK;
                continue;
              }

// Set a "ashadow copyed" flag

            if ((strcmp(poza,"shadow")) == 0)
              {
                scc2->_uflags._shadow = OK;
                continue;
              }

// All other variants we treat as codepage name, copy in codepages field, separate by
// space. We store comma BEFORE next codepage and only when first char isn't zero (this
// mean there is at least one codepage)

            if (*scc2->sccpages != 0)
              strcat(scc2->sccpages,",");

            strcat(scc2->sccpages,poza);
            scc2->_uflags._ischarset = OK;
          }

         scc2++;	// we cannot increment for empty structures :-)
       }		// Parsing host file main loop

#ifdef DBPRINT
      if (_SCC_milter_config.debug_level > 20)
        fprintf(stderr,
        	"%s %s SCC users file has been parsed, parsed %d records\n",
        						 localid,dblev[0],recs);
#endif

      free(bufx);
    }			// if ... open

// Specified configuration file didn't found

   else
     {
       fprintf(stderr,"opening file %s error: %s\n",filename,strerror(errno));
       return(NULL);
     }

  *qusers = recs;

// When we here, all OK

  return scc1;
}

/* ---------------------------------------------------------------------------------
 *
 *	Creating temp file (call, when created temp file to save)
 *
 *	Call with:	filename	- pointer to area with filename
 *			filedesc	- pointer to pointer to FILE to open
 *
 *	Return code:	zero 	 - error was detected, message arrived
 *			not zero - file was succesful created, desciptor stored
 *
 * --------------------------------------------------------------------------------- */

short int _SCC_milter_create_temp_file(char *filename, FILE **filedesc)
{
  int namelen;
  char *localid = "(utility)";
  char *slash;

// Serach back for first slash and check THIS directory (not dir + filename!)

  slash = strrchr(filename, '/');
  *slash = '\0';

// Check on presence work directory and correct permissions

  if (!(_SCC_milter_check_directory(filename)))
    return NO;
   else
     *slash = '/';

// Open and chmod temp file to keep header

  if ((namelen = mkstemp(filename)) == ERR )
    {
      syslog(LOG_ERR,"%s mkstemp error: %m, filename is %s", localid, filename);
      return NO;
    }

  if (chmod(filename, S_IWUSR | S_IRUSR | S_IRGRP | S_IWGRP) == ERR)
    syslog(LOG_ERR,"%s chmod error: %m, filename is %s", localid, filename);

// Associate file stream with opened descriptor

  if ((*filedesc = fdopen(namelen,"w+")) == NULL)
    {
      syslog(LOG_ERR,"%s fdopen error: %m, filename is %s", localid, filename);
      close(namelen);
      return NO;
    }

// Return succesful

  return OK;
}

/* ---------------------------------------------------------------------------------
 *
 *	Check on presence some directory and create it with 0700 rights
 *	(called before creating temp file or reciting message)
 *
 *	Call with:	pathname	- pointer to area with checking path
 *
 *	Return code:	zero 	 - error was detected, message arrived
 *			not zero - directory was succesfully created
 *
 * --------------------------------------------------------------------------------- */

int _SCC_milter_check_directory(char *pathname)
{
  struct stat sb;
  char *localid = "(utility)";

#ifdef DBPRINT
  if (_SCC_milter_config.debug_level > 50)
    syslog(LOG_ERR,"%s %s Checking directory %s",localid,dblev[1],pathname);
#endif

// Check on presence desired directory

  if ((stat(pathname, &sb)) == ERR)
    {

// When error is not "Directory not exist" - we warn and fall back

      if (errno != ENOENT)
        {
          syslog(LOG_ERR,"%s Error when tried to check presence %s - %m",
          							localid, pathname);
          return NO;
        }

#ifdef DBPRINT
      if (_SCC_milter_config.debug_level > 20)
        syslog(LOG_ERR,
        	"%s %s Directory %s will be created with 0700 rights",
        						 localid,dblev[0],pathname);
#endif

      if ((mkdir(pathname, 0700)) == ERR)
        {
          syslog(LOG_ERR,"%s Directory %s cannot be created - %m",localid,pathname);
          return NO;
        }
       else
         return OK;
    }
   else

// Check on desired rights on directory

     if ((!S_ISDIR(sb.st_mode) || ((access(pathname, S_IRWXU)) == ERR)))
       {

#ifdef DBPRINT
         if (_SCC_milter_config.debug_level > 20)
           syslog(LOG_ERR,
        	"%s %s Rights on directory %s will be reset to 0700",
        						 localid,dblev[0],pathname);
#endif

         chmod(pathname, S_IRWXU);
       }

  return OK;
}
